<?php
// $Id: random.php,v 1.1.1.1 2004/05/06 03:56:53 dyksjare Exp $
// =======================================================================
// Photoshare by Jorn Lind-Nielsen (C) 2002.
// ----------------------------------------------------------------------
// For POST-NUKE Content Management System
// Copyright (C) 2002 by the PostNuke Development Team.
// http://www.postnuke.com/
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WIthOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// =======================================================================

require_once("modules/photoshare/common.php");
require_once("modules/photoshare/pnblocks/common.php");

define('photoshareRandomBlockAll',    0);
define('photoshareRandomBlockLatest', 1);
define('photoshareRandomBlockTopic',  2);
define('photoshareRandomBlockFolder', 3);

/**
 * initialise block
 */
function photoshare_randomblock_init()
{
  // Security
  pnSecAddSchema('Photoshare:Randomblock:', 'Block title::Block Id');
}


/**
 * get information on block
 */
function photoshare_randomblock_info()
{
  // Values
  return array('text_type'      => 'photoshareRandom',
               'module'         => 'photoshare',
               'text_type_long' => 'Photoshare random image',
               'allow_multiple' => true,
               'form_content'   => false,
               'form_refresh'   => false,
               'show_preview'   => true);
}


function photoshare_randomblock_display($blockinfo)
{
    // Security check
  if (!pnSecAuthAction(0, 'photoshare:randomblock:', "$blockinfo[title]::$blockinfo[bid]", ACCESS_READ))
    return;

    // Get variables from content block
  $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Check refresh time (and existence of old content which is deleted on update of block settings)
  $past = time() - $blockinfo['refresh'];
  if($past < $blockinfo['unix_update']  &&  isset($vars['oldContent'])  &&  $vars['useRefreshTime'])
  {
      // No need to refresh - move old content into real content
    $blockinfo['content'] = $vars['oldContent'];
    return themesideblock($blockinfo);
  }

    // Database information
  pnModDBInfoLoad('photoshare');
  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

    // Do not know if necessary (some other part of PN might have done it - but better be sure)
  //srand(((double)microtime())*1000000);

  if (!pnModAPILoad('photoshare', 'user'))
    return "Failed to load Photoshare user API";

  if ($vars['type'] == photoshareRandomBlockFolder)
    $randomInfo = pnModAPIFunc('photoshare', 'user', 'get_random_image', array('folderID' => $vars['folder']));
  else if ($vars['type'] == photoshareRandomBlockTopic)
    $randomInfo = pnModAPIFunc('photoshare', 'user', 'get_random_image', array('topic' => $vars['topic']));
  else if ($vars['type'] == photoshareRandomBlockLatest)
    $randomInfo = pnModAPIFunc('photoshare', 'user', 'get_random_image', array('latest' => true));
  else
    $randomInfo = pnModAPIFunc('photoshare', 'user', 'get_random_image');

  $imageID  = $randomInfo['imageID'];
  $folderID = $randomInfo['folderID'];

  if (empty($imageID))
    return;

  $imageURL = pnModUrl('photoshare', 'user', 'viewimage', array('iid'       => $imageID,
                                                                'thumbnail' => 0 ));
  $thumbnailURL = pnModUrl('photoshare', 'user', 'viewimage', array('iid'       => $imageID,
                                                                    'thumbnail' => 1 ));
  $folderURL  = pnModUrl('photoshare', 'user', 'showimages', array('iid'      => $imageID,
                                                                   'fid'      => $folderID));
  $width    = ($vars['width']  ? $vars[width]  : "");
  $height   = ($vars['height'] ? $vars[height] : "");

    // Get image info
  $imageInfo = pnModAPIFunc('photoshare',
                            'user',
                            'get_image_info',
                            array( 'imageID' => $imageID ));

    // Get folder info
  $folderInfo = pnModAPIFunc('photoshare',
                             'user',
                             'get_folder_info',
                             array( 'folderID' => $folderID ));

    // Create the final HTML by substituting various macros into the user specified HTML code
  $substitutes = array( 'imageURL'      => $imageURL,
                        'thumbnailURL'  => $thumbnailURL,
                        'folderURL'     => $folderURL,
                        'title'         => $imageInfo['title'],
                        'owner'         => $imageInfo['ownername'],
                        'folderTitle'   => $folderInfo['title']);

  $html = $vars['html'];
//echo "X: $html"; exit(0);
  foreach ($substitutes as $key => $value)
  {
    $pattern = '${' . $key . '}';
    $html = str_replace($pattern, $value, $html);
  }

  $blockinfo['content'] = $html;
  $vars['oldContent'] = $html;
  $vars['previousImageID'] = $imageID;

    // Set "updated" value and store old content
  $blocksColumn = $pntable['blocks_column'];
  $sql = "UPDATE $pntable[blocks]
          SET    $blocksColumn[content]='" . pnVarPrepForStore(pnBlockVarsToContent($vars)) . "',
                 $blocksColumn[last_update] = " . $dbconn->DBTimestamp(time()) . "
          WHERE  $blocksColumn[bid] = " . pnVarPrepForStore($blockinfo['bid']);
  $result = $dbconn->Execute($sql);
  if($dbconn->ErrorNo() != 0)
    return "SQL error in Photoshare random block: " . $dbconn->errorMsg() . " while executing: $sql";

    // ... and return encapsulated in a theme block
  return themesideblock($blockinfo);
}


/**
 * modify block settings
 * - Current type [type]
 * -  Optional topic [topic]
 * -  Optional folder [folder]
 * - Fuld size or thumbnail [fullsize]
 * - width and height [width,height]
 */
function photoshare_randomblock_modify($blockinfo)
{
  $output = new pnHTML();

    // Get current content
  $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    // Empty fullsize and type cannot be distinguished from defaults, so do not set
  if (!isset($vars['folder']))
    $vars['folder'] = '';
  if (!isset($vars['topic']))
    $vars['topic'] = '';
  if (!isset($vars['html']))
    $vars['html'] = '<div class="photoshare-random-block"><a href="${imageURL}" target="_new"><img border="0" src="${thumbnailURL}"></a><br><b>${title}</b><br>Album: <a href="${folderURL}">${folderTitle}</a></div>';
  if (!isset($vars['useRefreshTime']))
    $vars['useRefreshTime'] = 0;

    // No need for a table start here since PN have supplied that already

    // Create row for help text
  $row = array();
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->SetOutputMode(_PNH_RETURNOUTPUT);
  $row[] = $output->Text(_PSRANDOMBLOCKHELP);
  $output->SetOutputMode(_PNH_KEEPOUTPUT);
  $output->SetInputMode(_PNH_PARSEINPUT);

    // Add row
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->TableAddRow($row, 'left');
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Create row for "All images"
  $row = array();
  $output->SetOutputMode(_PNH_RETURNOUTPUT);
  $row[] = $output->Text(_PSRANDOMBLOCKALLDESC);
  $row[] = $output->FormCheckbox('pstype', ($vars['type']==photoshareRandomBlockAll), photoshareRandomBlockAll, 'radio');
  $output->SetOutputMode(_PNH_KEEPOUTPUT);

    // Add row
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->TableAddRow($row, 'left');
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Create row for "Latest folder"
  $row = array();
  $output->SetOutputMode(_PNH_RETURNOUTPUT);
  $row[] = $output->Text(_PSRANDOMBLOCKLATESTDESC);
  $row[] = $output->FormCheckbox('pstype', ($vars['type']==photoshareRandomBlockLatest), photoshareRandomBlockLatest, 'radio');
  $output->SetOutputMode(_PNH_KEEPOUTPUT);

    // Add row
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->TableAddRow($row, 'left');
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Create row for "Images by topic"
  $topicList = photoshareBlockGetTopics($vars['topic']);
  if (is_bool($topicList))
    return photoshareBlockGetError();

  $row = array();
  $output->SetOutputMode(_PNH_RETURNOUTPUT);
  $row[] = $output->Text(_PSRANDOMBLOCKTOPICDESC);
  $row[] = $output->FormCheckbox('pstype', ($vars['type']==photoshareRandomBlockTopic), photoshareRandomBlockTopic, 'radio')
           . '&nbsp;' . $output->FormSelectMultiple('pstopic', $topicList)
           . '&nbsp;' . _PSRANDOMBLOCKTOPIC;
  $output->SetOutputMode(_PNH_KEEPOUTPUT);

    // Add row
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->TableAddRow($row, 'left');
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Create row for "Images from folder"
  $folderList = photoshareBlockGetFolderSelectList($vars['folder']);
  if (is_bool($folderList))
    return photoshareBlockGetError();

  $row = array();
  $output->SetOutputMode(_PNH_RETURNOUTPUT);
  $row[] = $output->Text(_PSRANDOMBLOCKFOLDERDESC);
  $row[] = $output->FormCheckbox('pstype', ($vars['type']==photoshareRandomBlockFolder), photoshareRandomBlockFolder, 'radio')
           . '&nbsp;' . $output->FormSelectMultiple('psfolder', $folderList)
           . '&nbsp;' . _PSRANDOMBLOCKFOLDER;
  $output->SetOutputMode(_PNH_KEEPOUTPUT);

    // Add row
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->TableAddRow($row, 'left');
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Add row for wrapper HTML description
  $output->TableRowStart();
  $output->TableColStart(1, 'left');
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->Text(_PSRANDOMBLOCKHTMLDESC 
                . ' (<a target="_new"'
                . ' href="modules/' . pnVarPrepForOS(photoshareModGetName()) . '/docs/randomblock/randomblock_examples.html">'
                . _PSRANDOMBLOCKHTMLEXAMPLES . '</a>):');
  $output->SetInputMode(_PNH_PARSEINPUT);
  $output->TableColEnd();
  $output->TableRowEnd();

    // Add row for wrapper HTML input
  $output->TableRowStart();
  $output->TableColStart(1, 'left');
  $output->FormTextArea('pshtml', $vars['html'], 8, 60);
  $output->SetOutputMode(_PNH_KEEPOUTPUT);
  $output->TableColEnd();
  $output->TableRowEnd();

    // Add row for wrapper HTML help
  $output->SetInputMode(_PNH_VERBATIMINPUT);

  $output->TableRowStart();
  $output->TableColStart(1, 'left');
  $output->Text(_PSRANDOMBLOCKHTMLHELP);
  $output->TableColEnd();
  $output->TableRowEnd();

  $output->TableRowStart();
  $output->TableColStart(1, 'left');

  $output->Text(  '<ul><li>${imageURL} - ' . _PSRANDOMBLOCKIMAGEURLHELP . "</li>\n"
                . '<li>${thumbnailURL} - ' . _PSRANDOMBLOCKTHUMBNAILURLHELP . "</li>\n"
                . '<li>${folderURL} - ' . _PSRANDOMBLOCKFOLDERURLHELP . "</li>\n"
                . '<li>${title} - ' . _PSRANDOMBLOCKTITLEHELP . "</li>\n"
                . '<li>${folderTitle} - ' . _PSRANDOMBLOCKFOLDERTITLEHELP . "</li>\n"
                . '<li>${owner} - ' . _PSRANDOMBLOCKOWNERHELP . "</li>\n"
                . "</ul>\n");
  
  $output->TableColEnd();
  $output->TableRowEnd();
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Create row for "depend on refresh time"
  $row = array();
  $output->SetOutputMode(_PNH_RETURNOUTPUT);
  $row[] = $output->Text(_PSRANDOMBLOCKREFRESHDESC);
  $row[] = $output->FormCheckbox('psUseRefreshTime', $vars['useRefreshTime'], 1);
  $output->SetOutputMode(_PNH_KEEPOUTPUT);

    // Add row
  $output->SetInputMode(_PNH_VERBATIMINPUT);
  $output->TableAddRow($row, 'left');
  $output->SetInputMode(_PNH_PARSEINPUT);


    // Return output
  return $output->GetOutput();
}


/**
 * update block settings
 */
function photoshare_randomblock_update($blockinfo)
{
  $vars = array('type'            => pnVarCleanFromInput('pstype'),
                'topic'           => pnVarCleanFromInput('pstopic'),
                'folder'          => pnVarCleanFromInput('psfolder'),
                'useRefreshTime'  => pnVarCleanFromInput('psUseRefreshTime'),
                'html'            => pnVarCleanFromInput('pshtml'));

  $blockinfo['content'] = pnBlockVarsToContent($vars);

  return $blockinfo;
}


?>
