<?php
// $Id: pnuserapi.php,v 1.1.1.1 2004/05/06 03:56:50 dyksjare Exp $
// =======================================================================
// Photoshare by Jorn Lind-Nielsen (C) 2002.
// ----------------------------------------------------------------------
// For POST-NUKE Content Management System
// Copyright (C) 2002 by the PostNuke Development Team.
// http://www.postnuke.com/
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WIthOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// =======================================================================

require_once("modules/photoshare/common.php");

define('photoshareDiskKindImage',     'img');
define('photoshareDiskKindThumbnail', 'tmb');


function photoshare_userapi_get_folder_info($args)
{
    // Argument check
  if (!isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_folder_info');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  $folderID         = $args['folderID'];
  $countSubFolders  = $args['countSubFolders'];
  $owner            = pnUserGetVar('uid');

     // Top most folder - return default info (FIXME: find or create top most folder)
  if ($folderID == -1)
    return true;

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = $pntable['photoshare_folders_column'];
  $imagesTable   = $pntable['photoshare_images'];
  $imagesColumn  = $pntable['photoshare_images_column'];
  $usersTable    = $pntable['users'];
  $usersColumn   = $pntable['users_column'];
  $topicsTable   = $pntable['topics'];
  $topicsColumn  = $pntable['topics_column'];

  if ($countSubFolders)
  {
    $subFolderSelect = ", COUNT(DISTINCT subFolders.ps_id)";
    $subFolderJoin   = "LEFT JOIN $foldersTable as subFolders ON subFolders.ps_parentfolder = " . pnVarPrepForStore($folderID);
  }
  else
  {
    $subFolderSelect = "";
    $subFolderJoin   = "";
  }

  $sql = "SELECT   $foldersColumn[id],
                   $foldersColumn[owner],
                   IF(LENGTH($usersColumn[name]) <> 0, $usersColumn[name], $usersColumn[uname]),
                   UNIX_TIMESTAMP($foldersColumn[createdDate]),
                   UNIX_TIMESTAMP($foldersColumn[modifiedDate]),
                   $foldersColumn[title],
                   $foldersColumn[description],
                   $foldersColumn[topic],
                   $topicsColumn[topicname],
                   $topicsColumn[topictext],
                   $foldersColumn[template],
                   $foldersColumn[hidepnframe],
                   $foldersColumn[blockfromlist],
                   $foldersColumn[parentFolder],
                   $foldersColumn[viewKey],
                   $foldersColumn[mainImage],
                   COUNT(DISTINCT $imagesColumn[id]),
                   MIN($imagesColumn[id])
                   $subFolderSelect
          FROM     $foldersTable 
                   LEFT JOIN $imagesTable ON $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID) . "
                   $subFolderJoin
                   LEFT JOIN $topicsTable ON $foldersColumn[topic] = $topicsColumn[topicid]
                   LEFT JOIN $usersTable  ON $foldersColumn[owner] = $usersColumn[uid]
          WHERE    $foldersColumn[id] = " . pnVarPrepForStore($folderID) . "
          GROUP BY $imagesColumn[parentFolder]";

  //if ($countSubFolders) { echo "<pre>$sql</pre>\n"; exit(0); }
  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_folder_info" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  if ($result->EOF)
    $folder = true; // SQL okay, but no data
  else
  {
    $mainImage = ($result->fields[15] == NULL ? $result->fields[17] : $result->fields[15]);
    $folder = array( 'id'            => $result->fields[0],
                     'owner'         => $result->fields[1],
                     'ownername'     => $result->fields[2],
                     'createdDate'   => strftime(_PSDATEFORMAT, $result->fields[3]),
                     'modifiedDate'  => strftime(_PSDATEFORMAT, $result->fields[4]),
                     'title'         => $result->fields[5],
                     'description'   => $result->fields[6],
                     'topic'         => $result->fields[7],
                     'topicName'     => $result->fields[8],
                     'topicText'     => $result->fields[9],
                     'template'      => $result->fields[10],
                     'hidepnframe'   => $result->fields[11],
                     'blockfromlist' => $result->fields[12],
                     'parentFolder'  => $result->fields[13],
                     'viewKey'       => $result->fields[14],
                     'mainImage'     => $mainImage,
                     'imageCount'    => $result->fields[16]);

    if ($countSubFolders)
      $folder['subFolderCount'] = $result->fields[18];
  }

  $result->Close();

  return $folder;
}


function photoshare_userapi_get_folder_info_rec($args)
{
    // Argument check
  if (    !isset($args['folderID'])
      ||  !isset($args['depth']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_folder_info_rec');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  $folderID = $args['folderID'];
  $depth    = $args['depth'];
  $viewKey  = $args['viewKey'];

  return photoshareGetFolderInfoRec($folderID, $viewKey, false, $depth);
}


function photoshareGetFolderInfoRec($folderID, $viewKey, $viewKeyFits, $depth)
{
  if ($depth <= 0)
    return array();

  if (!$viewKeyFits  &&  !photoshareAPIAccessFolder($folderID, photoshareAccessRequirementView, $viewKey))
    return false;

  $folder = photoshare_userapi_get_folder_info( array('folderID' => $folderID) );
  if (is_bool($folderInfo))
    return false;

  if ($viewKey != ''  &&  $folder['viewKey'] == $viewKey)
    $viewKeyFits = true;

  $images = photoshare_userapi_get_image_list( array('folderID' => $folderID) ); 
  if (is_bool($images))
    return false;
  $images = $images['images'];

  $folderSpecs = photoshare_userapi_get_folder_list( array('folderID'    => $folderID,
                                                           'ignoreOwner' => true) ); 
  $folders     = array();

  foreach ($folderSpecs as $folderSpec)
  {
    $nextFolderID = $folderSpec['id'];
    $folderInfo = photoshareGetFolderInfoRec($nextFolderID, $viewKey, $viewKeyFits, $depth-1);
    if ($folderInfo != false)
      $folders[] = $folderInfo;
  }

    // Create complete folder data
  $folderData = array( 'images'  => $images,
                       'folder'  => $folder,
                       'folders' => $folders );

  return $folderData;
}


function photoshare_userapi_get_folder_list($args)
{
    // Argument check
  if (!isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_folder_list');

  $folderID    = $args['folderID'];
  $ignoreOwner = $args['ignoreOwner'];
  $owner       = pnUserGetVar('uid');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = $pntable['photoshare_folders_column'];

  $ownerSQL = ($ignoreOwner ? "" : "AND  $foldersColumn[owner] = " . pnVarPrepForStore($owner));

  $sql = "SELECT   $foldersColumn[id],
                   $foldersColumn[title]
          FROM     $foldersTable
          WHERE    $foldersColumn[parentFolder] = " . pnVarPrepForStore($folderID) . "
          $ownerSQL
          ORDER BY $foldersColumn[title]";

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_folder_list" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $folders = array();

  for (; !$result->EOF; $result->MoveNext())
  {
    $folders[] = array('id'    => $result->fields[0],
                       'title' => $result->fields[1]);
  }

  $result->Close();

  return $folders;
}


function photoshare_userapi_get_folder_trail($args)
{
  if (!isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_folder_trail');

  $folderID = $args['folderID'];

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = $pntable['photoshare_folders_column'];

  $trail = array();

  if (!photoshareGetFolderTrailRec($dbconn, $foldersTable, $foldersColumn, $folderID, $trail))
    return false;

    // Add "top" link to trail
  $trail[] = array( 'id' => -1, 'title' => _PSTRAILTOP);

  return $trail;
}


function photoshareGetFolderTrailRec(&$dbconn, $foldersTable, $foldersColumn, $folderID, &$trail)
{
  $sql = "SELECT   $foldersColumn[parentFolder],
                   $foldersColumn[title]
          FROM     $foldersTable
          WHERE    $foldersColumn[id] = " . pnVarPrepForStore($folderID);

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"getFolderTrailRec" failed: ' . $dbconn->errorMsg() . " while executing: $sql");
  
  if ($result->EOF)
    return true;

  $trail[] = array('id'    => $folderID,
                   'title' => $result->fields[1]);

  $result->Close();

  if (!photoshareGetFolderTrailRec($dbconn, $foldersTable, $foldersColumn, $result->fields[0], $trail))
    return false;

  return true;
}


function photoshare_userapi_create_folder($args)
{
    // Argument check
  if (    !isset($args['parentFolderID'])
      ||  !isset($args['title'])
      ||  !isset($args['description'])
      ||  !isset($args['template']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_create_folder');

  $parentFolderID = $args['parentFolderID'];
  $title          = $args['title'];
  $description    = $args['description'];
  $topic          = $args['topic'];
  $template       = $args['template'];
  $hidepnframe   = $args['hidepnframe'];
  $blockfromlist = $args['blockfromlist'];
  $owner          = pnUserGetVar('uid');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

    // FIXME: what if not logged in - how about 'owner' ???

  $sql = "INSERT INTO $foldersTable (            
            $foldersColumn[owner],
            $foldersColumn[title],
            $foldersColumn[description],
            $foldersColumn[topic],
            $foldersColumn[template],
            $foldersColumn[hidepnframe],
            $foldersColumn[blockfromlist],
            $foldersColumn[parentFolder],
            $foldersColumn[createdDate],
            $foldersColumn[viewKey])
          VALUES (
            '" . pnVarPrepForStore($owner) . "',
            '" . pnVarPrepForStore($title) . "',
            '" . pnVarPrepForStore($description) . "',
            '" . pnVarPrepForStore($topic) . "',
            '" . pnVarPrepForStore(pnVarPrepForOS($template)) . "',
            '" . pnVarPrepForStore($hidepnframe) . "',
            '" . pnVarPrepForStore($blockfromlist) . "',
            '" . pnVarPrepForStore($parentFolderID) . "',
            NOW(),
            round(rand()*9000000000000 + 1000000000000))";
  
  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Create folder" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return $dbconn->insert_ID();
}


function photoshare_userapi_update_folder($args)
{
    // Argument check
    // Ignore hidepnframe and blockfromlist
  if (    !isset($args['folderID'])
      ||  !isset($args['title'])
      ||  !isset($args['description'])
      ||  !isset($args['template']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_update_folder');

  $folderID      = $args['folderID'];
  $title         = $args['title'];
  $description   = $args['description'];
  $topic         = $args['topic'];
  $template      = $args['template'];
  $hidepnframe   = $args['hidepnframe'];
  $blockfromlist = $args['blockfromlist'];

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

  $sql = "UPDATE $foldersTable SET
            $foldersColumn[title] = '" . pnVarPrepForStore($title) . "',
            $foldersColumn[description] = '" . pnVarPrepForStore($description) . "',
            $foldersColumn[topic] = '" . pnVarPrepForStore($topic) . "',
            $foldersColumn[template] = '" . pnVarPrepForStore(pnVarPrepForOS($template)) . "',
            $foldersColumn[hidepnframe] = '" . pnVarPrepForStore($hidepnframe) . "',
            $foldersColumn[blockfromlist] = '" . pnVarPrepForStore($blockfromlist) . "'
          WHERE $foldersColumn[id] = " . pnVarPrepForStore($folderID);

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Update folder" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return true;
}


function photoshare_userapi_delete_folder($args)
{
    // Argument check
  if (!isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_delete_folder');

  $folderID         = $args['folderID'];
  $deleteSubFolders = $args['deleteSubFolders'];

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];
  $imagesTable   = $pntable['photoshare_images'];
  $imagesColumn  = $pntable['photoshare_images_column'];

    // Delete(recursively) or move subfolders (if such exists)

  $ok = true;

  if ($deleteSubFolders)
  {
    $folders = photoshare_userapi_get_folder_list( array('folderID' => $folderID) );
    if (is_bool($folders)  &&  $folders == false)
      return false;

    foreach ($folders as $folder)
        // Recurse
      $ok = $ok && photoshare_userapi_delete_folder( array('folderID' => $folder['id'],
                                                           'deleteSubFolders' => true) );
  }
  else
  {
    $folder = photoshare_userapi_get_folder_info( array('folderID' => $folderID) );
    if (is_bool($folder)  &&  $folder == false)
      return false;

    $parentFolder = $folder['parentFolder'];

      // Set sub-folder's parent to current parent
    $sql = "UPDATE $foldersTable SET
              $foldersColumn[parentFolder] = " . pnVarPrepForStore($parentFolder) . "
            WHERE $foldersColumn[parentFolder] = " . pnVarPrepForStore($folderID);
    
    $dbconn->execute($sql);

    if ($dbconn->errorNo() != 0)
      return photoshareErrorAPI(__FILE__, __LINE__, '"Delete folder(1)" failed: ' . $dbconn->errorMsg() . " while executing: $sql");
  }

    // Delete images belonging to this folder
  if (pnModGetvar('photoshare', 'useimagedirectory'))
  {
      // Images in files - delete them one by one
    $images = photoshare_userapi_get_image_list( array('folderID' => $folderID) ); 
    if (is_bool($images)  &&  $images == false)
      return false;
    $images = $images['images'];

    foreach ($images as $image)
      $ok = $ok  &&  photoshare_userapi_delete_image( array('imageID' => $image['id']) );
  }

    // Delete database records for images
  $sql = "DELETE FROM $imagesTable
          WHERE $imagesColumn[parentFolder] = '" . pnVarPrepForStore($folderID) . "'";

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Delete folder(2)" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    // Delete access control records
  if (!photoshareClearAccessList($folderID))
    return false;

    // Delete the current folder
  $sql = "DELETE FROM $foldersTable
          WHERE $foldersColumn[id] = '" . pnVarPrepForStore($folderID) . "'";
  
  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Delete folder(4)" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return $ok;
}


// -----------------------------------------------------------------------
// User information
// -----------------------------------------------------------------------

function photoshare_userapi_get_user_info()
{
  $user = pnUserGetVar('uid');

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable  = $pntable['photoshare_images'];
  $imagesColumn = $pntable['photoshare_images_column'];

    // FIXME: what if not logged in - how about 'owner' ???

  $sql = "SELECT
            SUM($imagesColumn[byteSize])
          FROM $imagesTable 
          WHERE $imagesColumn[owner] = " . pnVarPrepForStore($user);
  
  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Get user info" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $userSetup = pnModAPIFunc('photoshare',
                            'user',
                            'get_user_setup',
                            array( 'uid' => $user ));

  return array( 'totalCapacityUsed'    => $result->fields[0],
                'imageSizeLimitSingle' => pnModGetVar('photoshare', 'imageSizeLimitSingle'),
                'imageSizeLimitTotal'  => $userSetup['storage'] );
}


function photoshare_userapi_get_user_setup($args)
{
  $user = $args['uid'];

    // Argument check
  if (!isset($args['uid']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_user_setup');

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $setupTable   = $pntable['photoshare_setup'];
  $setupColumn  = $pntable['photoshare_setup_column'];
  $groupmTable  = $pntable['group_membership'];
  $groupmColumn = $pntable['group_membership_column'];

  $sql = "SELECT MAX($setupColumn[storage])
          FROM   $setupTable
                 LEFT JOIN $groupmTable ON $setupColumn[id] = $groupmColumn[gid]                                           
          WHERE     ($setupColumn[kind] = " . photoshareUserSetupKind . "
                     AND $setupColumn[id] = " . pnVarPrepForStore($user) . ")
                 OR ($setupColumn[kind] = " . photoshareGroupSetupKind . "
                     AND $setupColumn[id] = $groupmColumn[gid]
                     AND $groupmColumn[uid] = " . pnVarPrepForStore($user) . ")";
  
  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Get user setup" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $storage = (is_null($result->fields[0])
                ? pnModGetVar('photoshare', 'imageSizeLimitTotal') 
                : $result->fields[0] * 1000); // Kilobytes!

  return array( 'storage' => $storage );
}


// -----------------------------------------------------------------------
// Images
// -----------------------------------------------------------------------

// Returns status array with the following layout:
// [
//   {
//     ok: boolean,
//     message: string (optional)
//   }
// ]
function photoshare_userapi_add_image($args)
{
  $dataFilename   = $args['dataFilename'];
  $imageName      = $args['imageFilename'];
  $mimeType       = $args['mimeType'];
  $isUploaded     = $args['isUploaded'];

  if (!isset($args['folderID'])  ||  $args['folderID'] < 0)
    return array( array('ok' => false,
                        'message' => photoshareErrorAPI(__FILE__, __LINE__, "Missing folder ID!") ) );

    // For OPEN_BASEDIR reasons we move the uploaded file as fast as possible to an accessible place
    // MUST remember to remove it afterwards!!!
  if ($isUploaded)
  {
      // Create and check tmpfilename
    $tmpdir = pnModGetVar('photoshare', 'tmpdirname');
    if (($tmpfilename = tempnam($tmpdir, 'IMG')) === false)
      return array( array('ok' => false,
                          'message' => "Unable to create tmpfilename in '$tmpdir' (adding image)" ) );

    if (move_uploaded_file($dataFilename, $tmpfilename) === false)
    {
      unlink($tmpfilename);
      return array( array('ok' => false,
                          'message' => "Unable to move uploaded file from '$dataFilename' to '$tmpfilename' (adding image)" ) );
    }

    $dataFilename = $tmpfilename;
    $args['dataFilename'] = $dataFilename;
  }

  $imageType = photoshareGetImageType($dataFilename, $imageName, $mimeType);

    // The mime type may be unknown when importing from local directory. So set it based on the image type
  if ($mimeType == 'unknown')
  {
    switch ($imageType)
    {
      case 'GIF': $mimeType='image/gif'; break;
      case 'JPG': $mimeType='image/jpeg'; break;
      case 'PNG': $mimeType='image/png'; break;
      default: $mimeType='image'; break;
    }

    $args['mimeType'] = $mimeType;
  }

	if ($imageType == 'ZIP')
    $result = photoshareAddZipArchive($args);
  else
    $result = array( photoshareAddSingleImage($args) );

    // Remove the created tmp file
  if ($isUploaded)
    unlink($dataFilename);

  return $result;
}


function photoshareAddSingleImage($args)
{
  //echo "F:$args[folderID],DF:$args[dataFilename],DS:$args[dataFileSize]";
  //echo "M:$args[mimeType],T:$args[title],D:$args[description]"; exit(0);

    // Argument check
  if (    !isset($args['folderID'])
      ||  !isset($args['dataFilename'])
      ||  !isset($args['dataFileSize'])
      ||  !isset($args['imageFilename'])
      ||  !isset($args['mimeType'])
      ||  !isset($args['title'])
      ||  !isset($args['description']))
    return array( 'ok'      => false,
                  'message' => _PSMODARGSERRORIN . ' photoshare_userapi_add_image' );

  $folderID      = $args['folderID'];
  $dataFilename  = $args['dataFilename'];
  $dataFileSize  = $args['dataFileSize'];
  $imageFilename = $args['imageFilename'];
  $mimeType      = $args['mimeType'];
  $title         = $args['title'];
  $description   = $args['description'];
  $isUploaded    = $args['isUploaded'];
  $owner         = pnUserGetVar('uid');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return array( 'ok'      => false,
                  'message' => _PSNOAUTH );

    // Ensure some name for the image
  if ($title == '')
  {
    $title = $imageFilename;
    if (!(($p=strrpos($title,'.')) === false))  // Strip trailing extension
      $title = substr($title,0,$p);
  }
    // Verify data file
  if ($dataFileSize == 0)
    return array( 'ok'      => false,
                  'message' => _PSMISSINGFILEFOR . " '$title'" );


    // Get user information
  $userInfo = pnModAPIFunc('photoshare',
                           'user',
                           'get_user_info');
  if ($userInfo === false)
    return array( 'ok'      => false,
                  'message' => photoshareErrorAPIGet() );

  $totalCapacityUsed = $userInfo['totalCapacityUsed'];


    // Check upload file size
  $totalCapacityUsed += $dataFileSize;

  if ($dataFileSize > $userInfo['imageSizeLimitSingle'])
    return array( 'ok'      => false,
                  'message' => _PSIMAGETOOBIG . "($imageFilename)" );

  if ($totalCapacityUsed > $userInfo['imageSizeLimitTotal'])
    return array( 'ok'      => false,
                  'message' => _PSCAPACITYLIMITREACHED . "($imageFilename)" );


    // Create image data
  $imageData = '';
  $thumbnailData = '';
  $optionalImageData = '';
  $optionalThumbnailData = '';
  $xSize = 0;
  $ySize = 0;
  $ok = createImageDataFromUpload($dataFilename, $imageFilename, $mimeType, $dataFileSize, $imageData, $thumbnailData,
                                  $optionalImageData, $optionalThumbnailData, $isUploaded, $xSize, $ySize);

  if (!$ok)
  {
    return array( 'ok'      => false,
                  'message' => photoshareErrorAPIGet() );
  }

    // Insert in database (optionally with image data)
  
  $newPosition  = photoshareGetNewImagePosition($folderID);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable  = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

    // Next step is the actual insertion of the image

  $sql = "INSERT INTO $imagesTable (            
            $imagesColumn[owner],
            $imagesColumn[filename],
            $imagesColumn[mimeType],
            $imagesColumn[createdDate],
            $imagesColumn[title],
            $imagesColumn[description],
            $imagesColumn[position],
            $imagesColumn[parentFolder],
            $imagesColumn[imageData],
            $imagesColumn[thumbnailData],
            $imagesColumn[xSize],
            $imagesColumn[ySize],
            $imagesColumn[byteSize])
          VALUES (
            '" . pnVarPrepForStore($owner) . "',
            '" . pnVarPrepForStore($imageFilename) . "',
            '" . pnVarPrepForStore($mimeType) . "',
            NOW(),
            '" . pnVarPrepForStore($title) . "',
            '" . pnVarPrepForStore($description) . "',
            $newPosition, 
            '" . pnVarPrepForStore($folderID) . "',
            '" . addslashes($optionalImageData) . "',
            '" . addslashes($optionalThumbnailData) . "',
            $xSize,
            $ySize,
            $dataFileSize)";

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return array( 'ok'      => false,
                  'message' => '"Add image" failed: ' . $dbconn->errorMsg() . " while executing: $sql" );

  if (pnModGetvar('photoshare', 'useimagedirectory'))
  {
    $id = $dbconn->Insert_ID();

    if (!photoshareDiskWriteImage($id, $imageData, $thumbnailData))
    {
        // Remove created database entry (where is the rollback functionality!)
      $sql = "DELETE FROM 
                $imagesTable
              WHERE $imagesColumn[id] = $id";
      $dbconn->execute($sql);

      return array( 'ok'      => false,
                    'message' => photoshareErrorAPIGet() );
    }
  }

  return array( 'ok'      => true,
                'message' => _PSADDINGIMAGE . " $title ($dataFileSize " . _PSBYTES . "): ");
}


function photoshareGetNewImagePosition($folderID)
{
  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable  = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

  $sql = "SELECT MAX($imagesColumn[position]), COUNT(*) FROM $imagesTable
          WHERE $imagesColumn[parentFolder] = '" . pnVarPrepForStore($folderID) . "'";

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"GetNewImagePosition" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  if ($result->EOF)
    $newPosition = 0;
  else
  {
    $count = $result->fields[1];
    if ($count == 0)
      $newPosition = 0;
    else
      $newPosition = intval($result->fields[0]) + 1;
  }

  return $newPosition;
}


function createImageDataFromUpload($dataFilename, $imageFilename, $mimeType, $dataFileSize, &$imageData, &$thumbnailData, 
                                   &$optionalImageData, &$optionalThumbnailData, $isUploaded, &$xSize, &$ySize)
{
  $imageType = photoshareGetImageType($dataFilename, $imageFilename, $mimeType);

    // Create image data
  if (!($f = fopen($dataFilename, 'rb')))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSERRORCREATINGIMAGE . "(3,$dataFilename)");

  $imageData = fread($f, $dataFileSize);

  fclose($f);
  
    // Create thumbnail data and get image size
  if (!($thumbnailData = photoshareCreateThumbnailData($dataFilename, $imageType, $isUploaded, $xSize, $ySize)))
    return false;

  if (pnModGetvar('photoshare', 'useimagedirectory'))
  {
    $optionalImageData = '';
    $optionalThumbnailData = '';
  }
  else
  {
    $optionalImageData = $imageData;
    $optionalThumbnailData = $thumbnailData;
  }

  return true;
}


function photoshare_userapi_get_image_info($args)
{
    // Argument check
  if (!isset($args['imageID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_image_info');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  $imageID = $args['imageID'];

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable   = $pntable['photoshare_images'];
  $imagesColumn  = $pntable['photoshare_images_column'];
  $usersTable    = $pntable['users'];
  $usersColumn   = $pntable['users_column'];

  $sql = "SELECT   $imagesColumn[id],
                   $imagesColumn[owner],
                   $usersColumn[name],
                   UNIX_TIMESTAMP($imagesColumn[createdDate]),
                   $imagesColumn[title],
                   $imagesColumn[description],
                   $imagesColumn[mimeType],
                   $imagesColumn[position],
                   $imagesColumn[parentFolder],
                   UNIX_TIMESTAMP($imagesColumn[modifiedDate]),
                   $imagesColumn[filename]
          FROM     $imagesTable,
                   $usersTable
          WHERE    $imagesColumn[id] = " . pnVarPrepForStore($imageID) .
         "  AND    $usersColumn[uid] = $imagesColumn[owner]";

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_image_info" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  if ($result->EOF)
    $image = false;
  else
    $image = array( 'id'            => $result->fields[0],
                    'owner'         => $result->fields[1],
                    'ownername'     => $result->fields[2],
                    'createdDate'   => $result->fields[3],
                    'title'         => $result->fields[4],
                    'description'   => $result->fields[5],
                    'mimeType'      => $result->fields[6],
                    'position'      => $result->fields[7],
                    'parentFolder'  => $result->fields[8],
                    'modifiedDate'  => $result->fields[9],
                    'filename'      => $result->fields[10]);


  $result->Close();

  return $image;
}


function photoshare_userapi_get_image_list($args)
{
    // Argument check
  if (!isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_image_list');

  $folderID     = $args['folderID'];
  $owner        = pnUserGetVar('uid');
  $page         = $args['page'];
  $usePageCount = $args['usePageCount'];

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable  = $pntable['photoshare_images'];
  $imagesColumn = $pntable['photoshare_images_column'];

  $sql = "SELECT   $imagesColumn[id],
                   $imagesColumn[title],
                   $imagesColumn[description],
                   $imagesColumn[position],
                   $imagesColumn[xSize],
                   $imagesColumn[ySize]
          FROM     $imagesTable
          WHERE    $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID) . "
          ORDER BY $imagesColumn[position]";

  $pageSize = photosharePageSize;
  $offset = $page * $pageSize;

  if ($usePageCount)
    $result = $dbconn->selectLimit($sql, $pageSize+1, $offset); // Select one extra to detect end of records
  else
    $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_image_list" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $images = array();
  $count = 0;

  for (; !$result->EOF; $result->MoveNext())
  {
    if ($count < $pageSize  ||  !$usePageCount)
    {
      $images[] = array('id'          => $result->fields[0],
                        'title'       => $result->fields[1],
                        'description' => $result->fields[2],
                        'position'    => $result->fields[3],
                        'xSize'       => $result->fields[4],
                        'ySize'       => $result->fields[5]);
    }

    ++$count;
  }

  $result->Close();

  $moreImages = ($count > $pageSize); // if count > pageSize then we have selected one more than we really asked for

  return array('images' => $images, 'moreImages' => $moreImages);
}


function photoshare_userapi_update_image($args)
{
    // Argument check
  if (    !isset($args['imageID'])
      ||  !isset($args['title'])
      ||  !isset($args['description']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_update_image');

  $imageID     = $args['imageID'];
  $title       = $args['title'];
  $description = $args['description'];
  $isUploaded  = $args['isUploaded'];

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

    // Update database (and files)

  $optionalImageData = '';
  $optionalThumbnailData = '';

  if ($args['dataFileSize'] > 0)
  {
      // User supplied new image file - make sure it overwrites the existing

    $dataFilename  = $args['dataFilename'];

      // For OPEN_BASEDIR reasons we move the uploaded file as fast as possible to an accessible place
      // MUST remember to remove it afterwards!!!
    $tmpdir = pnModGetVar('photoshare', 'tmpdirname');
    $tmpfilename = tempnam($tmpdir, 'IMG');
    if ($tmpfilename === false)
      return photoshareErrorAPI(__FILE__, __LINE__, "Unable to create tmpfilename in '$tmpdir' (update image)");
    if (move_uploaded_file($dataFilename, $tmpfilename) === false)
      return photoshareErrorAPI(__FILE__, __LINE__, "Unable to move uploaded file from '$dataFilename' to '$tmpfilename' (updating image)");

    $dataFilename = $tmpfilename;
    $args['dataFilename'] = $dataFilename;

    $dataFileSize  = $args['dataFileSize'];
    $imageFilename = $args['imageFilename'];
    $mimeType      = $args['mimeType'];

      // Create image data
    $imageData = '';
    $thumbnailData = '';
    $xSize = null;
    $ySize = null;
    $ok = createImageDataFromUpload($dataFilename, $imageFilename, $mimeType, $dataFileSize, $imageData, $thumbnailData,
                                    $optionalImageData, $optionalThumbnailData, $isUploaded, $xSize, $ySize);
    if (!$ok)
    {
      unlink($dataFilename);
      return false;
    }

      // Write (optionaly) image data to disc
    if (pnModGetvar('photoshare', 'useimagedirectory'))
    {
      if (!photoshareDiskWriteImage($imageID, $imageData, $thumbnailData))
      {
        unlink($dataFilename);
        return false;
      }
    }

    $imageUpdateSQL = ", $imagesColumn[imageData] = '" . addslashes($optionalImageData) . "',
                         $imagesColumn[thumbnailData] = '" . addslashes($optionalThumbnailData) . "',
                         $imagesColumn[mimeType] = '" . pnVarPrepForStore($mimeType) . "'";

    unlink($dataFilename);
  }
  else
    $imageUpdateSQL = '';

  if ($xSize != null  &&  $ySize != null)
    $sizeSQL = ", $imagesColumn[xSize] = '" . pnVarPrepForStore($xSize) . "',
                  $imagesColumn[ySize] = '" . pnVarPrepForStore($ySize) . "'";
  else
    $sizeSQL = '';
            
  $sql = "UPDATE $imagesTable SET
            $imagesColumn[title] = '" . pnVarPrepForStore($title) . "',
            $imagesColumn[description] = '" . pnVarPrepForStore($description) . "'
            $sizeSQL
            $imageUpdateSQL
          WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);
  
  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Update image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return true;
}


function photoshare_userapi_move_image($args)
{
    // Argument check
  if (    !isset($args['imageID'])
      ||  !isset($args['position']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_move_image');

  $imageID     = $args['imageID'];
  $dstPosition = $args['position'];

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // Get image information
  $imageInfo    = photoshare_userapi_get_image_info( array('imageID' => $imageID) );
  $folderID     = $imageInfo['parentFolder'];
  $srcPosition  = $imageInfo['position'];

    // If moving from left to right then adjust destition one to the left
  if ($srcPosition < $dstPosition)
    --$dstPosition;

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

    // First update moves all images after current image one to the left.
  if (!photoshareShiftImagesLeft($dbconn, $folderID, $srcPosition+1, null, 1))
    return false;

    // Next update moves all images after new position one to the right.

  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

  $sql = "UPDATE $imagesTable SET
            $imagesColumn[position] = $imagesColumn[position] + 1
          WHERE     $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID) .
         "      AND $imagesColumn[position] >= " . pnVarPrepForStore($dstPosition);
  
  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Move image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    // Last update moves image to new position

  $sql = "UPDATE $imagesTable SET
            $imagesColumn[position] = " . pnVarPrepForStore($dstPosition) .
        " WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);
  
  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Move image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return true;
}


function photoshare_userapi_move_image_to_folder($args)
{
    // Argument check
  if (   !isset($args['imageID'])
      || !isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_move_image_to_folder');

  $imageID      = $args['imageID'];
  $dstFolderID  = $args['folderID'];

    // Check edit access for source image and destination folder
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit, ''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);
  if (!photoshareAPIAccessFolder($dstFolderID, photoshareAccessRequirementEdit, ''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // Get image information (especially parent folder ID)
  $imageInfo = pnModAPIFunc('photoshare',
                            'user',
                            'get_image_info',
                            array( 'imageID' => $imageID ));
  if (is_bool($imageInfo)  &&  $imageInfo == false)
    return false;

    // Do the image move (just changing parent folder ID)
  
  $newPosition = photoshareGetNewImagePosition($dstFolderID);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

  $sql = "UPDATE $imagesTable SET
            $imagesColumn[parentFolder] = '" . pnVarPrepForStore($dstFolderID) . "',
            $imagesColumn[position] = $newPosition
          WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Move image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    // Update image positions in origin folder
  if (!photoshareShiftImagesLeft($dbconn, $imageInfo['parentFolder'], $imageInfo['position']+1, null, 1))
    return false;

  return true;
}


function photoshareShiftImagesLeft(&$dbconn, $folderID, $beginPosition, $endPosition, $offset)
{
  $pntable = pnDBGetTables();
  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

  $sql = "UPDATE $imagesTable SET
            $imagesColumn[position] = $imagesColumn[position] - $offset
          WHERE     $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID);

  if ($beginPosition != null)
    $sql .= "      AND $imagesColumn[position] >= " . pnVarPrepForStore($beginPosition) . "\n";

  if ($endPosition != null)
    $sql .= "      AND $imagesColumn[position] < " . pnVarPrepForStore($endPosition);

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Shift image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return true;
}


function photoshare_userapi_copy_image_to_folder($args)
{
    // Argument check
  if (   !isset($args['imageID'])
      || !isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_copy_image_to_folder');

  $imageID  = $args['imageID'];
  $folderID = $args['folderID'];

    // Check edit access for source image and destination folder
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit, ''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);
  if (!photoshareAPIAccessFolder($folderID, photoshareAccessRequirementEdit, ''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // Do the image copy
  
    // Get basic image information
  $imageInfo = pnModAPIFunc('photoshare',
                            'user',
                            'get_image_info',
                            array( 'imageID' => $imageID ));
  if (is_bool($imageInfo)  &&  $imageInfo == false)
    return false;

    // Build image files to make it look like a file upload
  if (pnModGetvar('photoshare', 'useimagedirectory'))
  {
      // Images on disk - just refer to the database file
    $dataFilename = photoshareDiskImageFilename(photoshareDiskKindImage, $imageID);
    $dataFileSize = filesize($dataFilename);
  }
  else
  {
      // Images in database - export image data to a tmp. file

      // Get tmp. filename
    $tmpdir = pnModGetVar('photoshare', 'tmpdirname');
    $dataFilename = tempnam($tmpdir, 'IMG');
    if ($dataFilename === false)
      return photoshareErrorAPI(__FILE__, __LINE__, "Unable to create tmpfile in '$tmpdir' (copy_image_to_folder)");

      // Export image data
    $imageData = photoshareGetImageData($imageID, false, true);
    $f = fopen($dataFilename, "wb");
    if (!$f)
      return photoshareErrorAPI(__FILE__, __LINE__, "Unable to open tmp. file '$dataFilename' for write");
    fwrite($f, $imageData['imageData']);
    fclose($f);

    $dataFileSize = strlen($imageData['imageData']);
  }

  $statusSet = pnModAPIFunc('photoshare',
                            'user',
                            'add_image',
                            array( 'folderID'      => $folderID,
                                   'dataFilename'  => $dataFilename,
                                   'dataFileSize'  => $dataFileSize,
                                   'imageFilename' => $imageInfo['filename'],
                                   'mimeType'      => $imageInfo['mimeType'],
                                   'title'         => $imageInfo['title'],
                                   'description'   => $imageInfo['description'],
                                   'isUploaded'    => false ));

  if (!pnModGetvar('photoshare', 'useimagedirectory'))
  {
      // Remove the temporary image file used when storing images in the database
    unlink($dataFilename);
  }

  return $statusSet[0]['ok'];
}


function photoshare_userapi_set_main_image($args)
{
    // Argument check
  if (!isset($args['imageID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_set_main_image');

  $imageID = $args['imageID'];

    // Check access
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit, ''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  $imageInfo = photoshare_userapi_get_image_info(array( 'imageID' => $imageID ));
  $folderID  = $imageInfo['parentFolder'];

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

  $sql = "UPDATE
            $foldersTable
          SET
            $foldersColumn[mainImage] = " . pnVarPrepForStore($imageID) . "
          WHERE $foldersColumn[id] = " . pnVarPrepForStore($folderID);

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, "'SetMainImage' failed while executing: $sql");

  return true;
}


function photoshare_userapi_delete_image($args)
{
    // Argument check
  if (!isset($args['imageID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_delete_image');

  $imageID = $args['imageID'];

    // Check access
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit, ''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  $imageInfo = photoshare_userapi_get_image_info(array( 'imageID' => $imageID ));
  $folderID  = $imageInfo['parentFolder'];
  $position  = $imageInfo['position'];

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

    // Delete the actual image

  $sql = "DELETE FROM 
            $imagesTable
          WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);
  
  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Delete image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    // Delete on disc - ignore missing files
  if (pnModGetvar('photoshare', 'useimagedirectory'))
    photoshareDiskDeleteImage($imageID);

    // Update position of remaining images in folder

  $sql = "UPDATE $imagesTable SET
            $imagesColumn[position] = $imagesColumn[position] - 1
          WHERE     $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID) .
         "      AND $imagesColumn[position] > " . pnVarPrepForStore($position);

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Delete image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    // Update optional main image in parent folder

  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

  $sql = "UPDATE $foldersTable
            SET $foldersColumn[mainImage] = NULL
          WHERE $foldersColumn[mainImage] = " . pnVarPrepForStore($imageID);

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Delete image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return true;
}


function photoshare_userapi_rotate_image($args)
{
    // Argument check
  if (    !isset($args['imageID'])
      &&  !isset($args['direction']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_rotate_image');

  $imageID   = $args['imageID'];
  $direction = $args['direction'];
  $owner     = pnUserGetVar('uid');

    // Check access
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit,''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // Retrieve both image and thumbnail data
  $data = photoshareGetImageData($imageID, true, true);
  if ($data === false)
    return false;

  $rotatedImageData     = photoshareRotateImageFromString($data['imageData'], $direction);
  $rotatedThumbnailData = photoshareRotateImageFromString($data['thumbnailData'], $direction);

  photoshareUpdateImageData($imageID, $rotatedImageData, $rotatedThumbnailData, true);

  return true;
}


function photoshare_userapi_recalc_thumbnail($args)
{
    // Argument check
  if (!isset($args['imageID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_recalc_thumbnail');

  $imageID = $args['imageID'];

    // Check access
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit,''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // Retrieve both image and thumbnail data
  $data = photoshareGetImageData($imageID, true, true);
  if ($data === false)
    return false;

  $im = imagecreatefromstring($data['imageData']);
  $thumbnailData = photoshareCreateThumbnailFromGD($im);

  photoshareUpdateImageData($imageID, null, $thumbnailData, false);

  return true;
}


function photoshare_userapi_recalc_size($args)
{
    // Argument check
  if (!isset($args['imageID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_recalc_thumbnail');

  $imageID = $args['imageID'];

    // Check access
  if (!photoshareAPIAccessImage($imageID, photoshareAccessRequirementEdit,''))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // Retrieve image data
  $data = photoshareGetImageData($imageID, false, true);
  if ($data === false)
    return false;

  $im = imagecreatefromstring($data['imageData']);

  if ($im === false)
    photoshareErrorAPI(__FILE__, __LINE__, "Unable to create image '$imageID' from data while recalculating sizes.");

  $result = array('xSize' => imagesx($im), 'ySize' => imagesy($im));

  imagedestroy($im);

  return $result;
}


function photoshare_userapi_get_random_image($args)
{
  $orgFolderID = (isset($args['folderID']) ? $args['folderID'] : null);
  $topic       = (isset($args['topic']) ? $args['topic'] : null);
  $latest      = (isset($args['latest']) ? $args['latest'] : false);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = $pntable['photoshare_images_column'];

  $user = pnUserGetVar('uid');
  
  $args = array('user'       => $user,
                'getForList' => true,
                'topic'      => $topic);

  if ($latest)
    $args['order'] = 'date';

  $folders = pnModAPIFunc('photoshare', 'user', 'get_accessible_folders', $args);

  if ($folders === false || count($folders) == 0)
    return false;

  $count = 0;

    // Get random folder if not specified - repeat until one with images in it is found
  do
  {
    if ($orgFolderID == null)
    {
      if ($latest)
        $folderID = $folders[0]['id'];
      else
        $folderID = $folders[rand(0, sizeof($folders)-1)]['id'];
    }
    else
      $folderID = $orgFolderID;

      // Now we have a folder - so find an image in this folder

      // Check access for this folder (could have been specified directly)
    $accessOk = pnModAPIFunc('photoshare', 'user', 'get_folder_access', 
                             array('folderID'    => $folderID,
                                   'accessLevel' => $photoshareAccessRequirementView,
                                   'viewKey'     => null) );

    if (!$accessOk)
      continue; // Keep looking for a new folder

      // Extract number of images in this folder

    $sql = "SELECT count(*)
            FROM $imagesTable
            WHERE $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID);

    $result = $dbconn->execute($sql);

    if ($dbconn->errorNo() != 0)
      return photoshareErrorAPI(__FILE__, __LINE__, '"get_random_image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    $maxImageCount = $result->fields[0];

    $result->Close();
  }
  while ($maxImageCount == 0  &&  $count++ < 10);

  if ($maxImageCount == 0)
    return photoshareErrorAPI(__FILE__, __LINE__, 'Could not find a random image for current user');

  $imageCount = rand(1, $maxImageCount);

    // Select the image that corresponds to the sequence number
  $sql = "SELECT $imagesColumn[id]
          FROM $imagesTable
          WHERE $imagesColumn[parentFolder] = " . pnVarPrepForStore($folderID);

  $result = $dbconn->selectLimit($sql, 1, $imageCount-1);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_random_image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $imageID = $result->fields[0];

  $result->Close();

  return array('folderID' => $folderID, 'imageID' => $imageID);
}


  // Output image directly to stdout
function photoshare_userapi_dumpimage($args)
{
    // Argument check
  if (!isset($args['imageID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_dumpimage');

  $imageID   = $orgImageID = $args['imageID'];
  $thumbnail = $args['thumbnail']; // optional bool
  $doCache   = (isset($args['cache']) ? $args['cache'] : true);
  $owner     = pnUserGetVar('uid');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);  // FIXME: output 'no access' image

    // Some users have reported this to make their setup work. The buffer may contain something
    // due to a buggy template or block
  while (@ob_end_clean());

    // Check browser cache date

    // Retrieve image information
  $imageInfo = photoshare_userapi_get_image_info(array( 'imageID' => $imageID ));
  if ($imageInfo === false)
    return photoshareErrorAPI(__FILE__, __LINE__, "Unknown image");

    // Check cached versus modified date

  $lastModifiedDate = date('D, d M Y H:i:s T', $imageInfo[modifiedDate]);
  $currentETag = $imageInfo[modifiedDate];

  global $HTTP_SERVER_VARS;
  $cachedDate = $HTTP_SERVER_VARS['HTTP_IF_MODIFIED_SINCE'];
  $cachedETag = $HTTP_SERVER_VARS['HTTP_IF_NONE_MATCH'];

    // If magic quotes are on then all query/post variables are escaped - so strip slashes to make a compare possible
  if (get_magic_quotes_gpc())
    $cachedETag = stripslashes($cachedETag);

  if (    (empty($cachedDate) || $lastModifiedDate == $cachedDate)
      &&  '"'.$currentETag.'"' == $cachedETag
      &&  $doCache)
  {
    header("HTTP/1.1 304 Not Modified");
    header("Expires: " . date('D, d M Y H:i:s T', time()+180*24*3600)); // My PHP insists on Expires in 1981 as default!
  	header('Pragma: cache'); // My PHP insists on putting a pragma "no-cache", so this is an attempt to avoid that
    header("ETag: \"$imageInfo[modifiedDate]\"");
    return true;
  }

  header("Expires: " . date('D, d M Y H:i:s T', time()+180*24*3600)); // My PHP insists on Expires in 1981 as default!
	header('Pragma: cache'); // My PHP insists on putting a pragma "no-cache", so this is an attempt to avoid that
  header("ETag: \"$imageInfo[modifiedDate]\"");

    // Retrieve image data from repository

    // When loading images from directory it is better just to dump them directly, so ignore standard fetch
  if (!pnModGetvar('photoshare', 'useimagedirectory'))
  {
    $data = photoshareGetImageData($imageID, $thumbnail, !$thumbnail);
    
    if (is_bool($data)  &&  $data == false)
      return false;
  }

    // Setup output and dump the image

    // Ensure correct content-type and a filename for eventual download
  if ($thumbnail)
 	  header("Content-Type: image/png");
  else
 	  header("Content-Type: $imageInfo[mimeType]");
  header("Content-Disposition: inline; filename=\"$imageInfo[title]\"");
	header("Last-Modified: $lastModifiedDate");
  
  if (pnConfigGetVar('UseCompression') == 1)
    header("Content-Encoding: gzip");

      // When loading images from directory it is better just to dump them directly, so ignore standard fetch
  if (pnModGetvar('photoshare', 'useimagedirectory'))
  {
    $filename = photoshareDiskImageFilename( ($thumbnail ? photoshareDiskKindThumbnail : photoshareDiskKindImage), $imageID);

    header("Content-Length: " . filesize($filename));
    readfile($filename);
  }
  else
  {
    if ($thumbnail)
    {
      header("Content-Length: " . strlen($data['thumbnailData']));
      echo $data['thumbnailData'];
    }
    else
    {
      header("Content-Length: " . strlen($data['imageData']));
      echo $data['imageData'];
    }
  }

  return true;
}


function photoshareDiskImageFilename($kind, $id)
{
  return pnModGetVar('photoshare', 'imagedirname') . '/' . $kind . $id;
}


function photoshareDiskWriteImage($id, $imageData, $thumbnailData)
{
  $ok = true;

  if ($imageData != null)
    $ok = $ok && photoshareDiskWriteImage2(photoshareDiskKindImage, $id, $imageData);

  if ($thumbnailData != null)
    $ok = $ok && photoshareDiskWriteImage2(photoshareDiskKindThumbnail, $id, $thumbnailData);

  return $ok;
}


function photoshareDiskWriteImage2($kind, $id, $data)
{
  $filename = photoshareDiskImageFilename($kind, $id);

  $f = fopen($filename, 'wb');
  if (!$f)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Add image" failed: could not open file "' . $filename . '" for writing.');

  fwrite($f, $data);
  fclose($f);

  if (ini_get('safe_mode'))
    chmod($filename, 0777); // Make sure it is accessible from FTP when running strict mode

  return true;
}


function photoshareDiskDeleteImage($id)
{
  $ok = true;

  $filename = photoshareDiskImageFilename(photoshareDiskKindImage, $id);
  if (!@unlink($filename))
    $ok = photoshareErrorAPI(__FILE__, __LINE__, '"Delete image" failed: could not unlink file "' . $filename . '".');

  $filename = photoshareDiskImageFilename(photoshareDiskKindThumbnail, $id);
  if (!@unlink($filename))
    $ok = $ok && photoshareErrorAPI(__FILE__, __LINE__, '"Delete image" failed: could not unlink file "' . $filename . '".');

  return $ok;
}


function photoshareDiskReadImage($kind, $id)
{
  $filename = photoshareDiskImageFilename($kind, $id);
  if (($f = fopen($filename, 'rb')) == false)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Read image" failed: could not open file "' . $filename . '" for read.');
  $data = fread($f, filesize($filename));
  fclose($f);

  return $data;
}


function photoshareGetImageData($imageID, $getThumbnail, $getImage)
{
  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $useDirectory = pnModGetvar('photoshare', 'useimagedirectory');

  $imagesTable  = $pntable['photoshare_images'];
  $imagesColumn = $pntable['photoshare_images_column'];

  $thumbnailField = ($getThumbnail && !$useDirectory ? $imagesColumn['thumbnailData'] : '0');
  $imageField     = ($getImage && !$useDirectory     ? $imagesColumn['imageData'] : '0');

  $sql = "SELECT   $imageField,
                   $thumbnailField,
                   $imagesColumn[mimeType],
                   $imagesColumn[title],
                   $imagesColumn[modifiedDate]
          FROM     $imagesTable
          WHERE    $imagesColumn[id] = " . pnVarPrepForStore($imageID);

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"getimagedata" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  if ($useDirectory)
  {
    $imageData     = ($getImage     ? photoshareDiskReadImage(photoshareDiskKindImage, $imageID) : '0');
    $thumbnailData = ($getThumbnail ? photoshareDiskReadImage(photoshareDiskKindThumbnail, $imageID) : '0');

    if (($getImage  &&  $imageData == false)  ||  ($getThumbnail  &&  $thumbnailData == false))
      return false;
  }
  else
  {
    $imageData = $result->fields[0];
    $thumbnailData = $result->fields[1];
  }

  $data = array( 'imageData'     => $imageData,
                 'thumbnailData' => $thumbnailData,
                 'mimeType'      => $result->fields[2],
                 'title'         => $result->fields[3],
                 'modifiedDate'  => $result->fields[4]);

  $result->Close();

  return $data;
}


  // Update image data - either on disk or in database - but always update modified date in database.
function photoshareUpdateImageData($imageID, $imageData, $thumbnailData, $rotated = false)
{
  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $imagesTable = $pntable['photoshare_images'];
  $imagesColumn = &$pntable['photoshare_images_column'];

  if ($rotated)
  {
      // I don't know how to swap two values in mySQL, so pick out and update afterwards

    $sql = "SELECT $imagesColumn[xSize], $imagesColumn[ySize] from $imagesTable
            WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);

    $result = $dbconn->execute($sql);

    if ($dbconn->errorNo() != 0)
      return photoshareErrorAPI(__FILE__, __LINE__, "'photoshareUpdateImageData' failed while executing: $sql");

    if (!$result->EOF)
    {
      $xSize = $result->fields[0];
      $ySize = $result->fields[1];
    }

    $rotateSQL =  ", $imagesColumn[xSize] = $ySize, $imagesColumn[ySize] = $xSize";
  }
  else
    $rotateSQL = '';

  if (pnModGetvar('photoshare', 'useimagedirectory'))
  {
      // Update at least x/y sizes (and timestamp at the same time)
    $sql = "UPDATE $imagesTable SET 
              $imagesColumn[modifiedDate] = NULL
              $rotateSQL
            WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);

      // Write actual data to disk
    $ok = photoshareDiskWriteImage($imageID, $imageData, $thumbnailData);
  }
  else
  {
      // Update both modified timestamp (automatic) and image data
    $sql = "UPDATE $imagesTable SET ";

    if ($imageData != null)
      $sql .= "$imagesColumn[imageData] = '" . addslashes($imageData) . "'";

    if ($thumbnailData != null  &&  $imageData != null)
      $sql .= ', ';

    if ($thumbnailData != null)
      $sql .= "$imagesColumn[thumbnailData] = '" . addslashes($thumbnailData) . "'";

    $sql .= $rotateSQL;
    $sql .=" WHERE $imagesColumn[id] = " . pnVarPrepForStore($imageID);

    $ok = true;
  }

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Update rotated image" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return $ok;
}


function photoshareGetImageType($dataFilename, $imageFilename, $mimeType)
{
	if ($mimeType == 'application/x-zip-compressed' ||
	    $mimeType == 'application/x-compressed' ||
	    $mimeType == 'application/zip' ||
	    $mimeType == 'multipart/x-zip')
    return 'ZIP';

  if (strlen($imageFilename) >= 4)
    if (stristr(substr($imageFilename,-4), '.zip') == '.zip')
      return 'ZIP';

  $gdSizeInfo = getimagesize($dataFilename);

  switch ($gdSizeInfo[2])
  {
    case  0: return 'unknown';
    case  1: return 'GIF';
    case  2: return 'JPG';
    case  3: return 'PNG';
    case  4: return 'SWF';
    case  5: return 'PSD';
    case  6: return 'BMP';
    case  7: return 'TIFF';
    case  8: return 'TIFF';
    default: return 'OTHER';
  }

  return $mimeType;
}


function photoshareCreateImageFromFile($filename, $imageType)
{
  $im = false;

  if ($imageType == 'JPG')
  {
    $im = @imagecreatefromjpeg($filename);
  }
  else if ($imageType == 'GIF')
  {
    $im = @imagecreatefromgif($filename);
  }
  else if ($imageType == 'PNG')
  {
    $im = @imagecreatefrompng($filename);
  }
  else
    return photoshareErrorAPI(__FILE__, __LINE__, _PSUNKNOWNIMAGEFORMAT . " '$imageType' (" . $filename . ")");

  if (!$im)
    return photoshareErrorAPI(__FILE__, __LINE__, _PSUNSUPPORTEDIMAGEFORMAT . " '$imageType' (" . $filename . ")");

  return $im;
}


function photoshareCreateImageDirect($xsize, $ysize, $imageType, &$isTrueColor)
{
    // Detect presense of ImageCreateTrueColor() - although this apparently doesn't say anything about GD2 existing ...
  $hasTrueColorImage = false;
  $gdfunctions = get_extension_funcs("gd");
  foreach ($gdfunctions as $f)
    if ($f == 'imagecreatetruecolor')
      $hasTrueColorImage = true;

    // If the check for imagecreatetruecolor fails then insert the next line
  // $hasTrueColorImage = false;

  $isTrueColor = false;

    // Create actual image
  if ($imageType == 'GIF'  ||  !$hasTrueColorImage)
    $image = ImageCreate($xsize,$ysize); // Create white background image
  else
  {
    $image = @ImageCreateTrueColor($xsize,$ysize); // Create black background image

      // Didn't work ... perhaps ImageCreateTrueColor doesn't exist?
    if (!$image)
      $image = ImageCreate($xsize,$ysize); // Create white background image
    else
      $isTrueColor = true;
  }

  return $image;
}

/* This functions allows the resize images bicubic without GD2 */
function photoshareImageCopyResampleBicubic ($dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h)
{
	/*
	 * port to PHP by John Jensen July 10 2001 -- original code (in C, for the PHP GD Module) by jernberg@fairytale.se
  */

	// Left out code for indexed images
  $scaleX = ($src_w - 1) / $dst_w;
  $scaleY = ($src_h - 1) / $dst_h;

  $scaleX2 = $scaleX / 2.0;
  $scaleY2 = $scaleY / 2.0;

  for ($j = $src_y; $j < $dst_h; $j++)
  {
	  $sY = $j * $scaleY;
    for ($i = $src_x; $i < $dst_w; $i++)
  	{
    	$sX = $i * $scaleX;

      $c1 = ImageColorsForIndex ($src_img, ImageColorAt ($src_img, (int) $sX, (int) $sY + $scaleY2));
      $c2 = ImageColorsForIndex ($src_img, ImageColorAt ($src_img, (int) $sX, (int) $sY));
      $c3 = ImageColorsForIndex ($src_img, ImageColorAt ($src_img, (int) $sX + $scaleX2, (int) $sY + $scaleY2));
      $c4 = ImageColorsForIndex ($src_img, ImageColorAt ($src_img, (int) $sX + $scaleX2, (int) $sY));

      $red = (int) (($c1['red'] + $c2['red'] + $c3['red'] + $c4['red']) / 4);
	    $green = (int) (($c1['green'] + $c2['green'] + $c3['green'] + $c4['green']) / 4);
      $blue = (int) (($c1['blue'] + $c2['blue'] + $c3['blue'] + $c4['blue']) / 4);

      $color = ImageColorClosest ($dst_img, $red, $green, $blue);
      ImageSetPixel ($dst_img, $i + $dst_x, $j + $dst_y, $color);
    }
  }
}

function photoshareCreateThumbnailData($filename, $imageType, $isUploaded, &$xSize, &$ySize)
{
  $im = photoshareCreateImageFromFile($filename, $imageType);

  if (!$im)
    return false;

  $xSize = imagesx($im);  
  $ySize = imagesy($im);  

  return photoshareCreateThumbnailFromGD($im);
}


function photoshareCreateThumbnailFromGD($im)
{
  $addSquareBorder = false;

  $thumbnailSize = pnModGetVar('photoshare', 'thumbnailsize');
  $thumbnailInnerSize = ($addSquareBorder
                         ? ($thumbnailSize > 4 ? $thumbnailSize - 4 : $thumbnailSize)
                         : $thumbnailSize);

  $xs = imagesx($im);
  $ys = imagesy($im);

    // Calculate thumbnail X and Y sizes
  if ($xs > $ys)
  {
    $thumbnailXSize = $thumbnailInnerSize;
    $thumbnailYSize = ($ys*$thumbnailInnerSize) / $xs;
  }
  else
  {
    $thumbnailYSize = $thumbnailInnerSize;
    $thumbnailXSize = ($xs*$thumbnailInnerSize) / $ys;
  }

  if ($addSquareBorder)
  {
      // Calculate X,Y offset for centering in thumbnail
    $xoffset = ($thumbnailSize - $thumbnailXSize) / 2;
    $yoffset = ($thumbnailSize - $thumbnailYSize) / 2;
  }
  else
  {
    $xoffset = $yoffset = 0;
  }

  $isTrueColor = false;

  if ($addSquareBorder)
    $thumbnail = photoshareCreateImageDirect($thumbnailSize, $thumbnailSize, $imageType, $isTrueColor);
  else
    $thumbnail = photoshareCreateImageDirect($thumbnailXSize, $thumbnailYSize, $imageType, $isTrueColor);

  $white = imagecolorallocate($thumbnail, 255, 255, 255); // First allocated - becomes background
  $gray = imagecolorallocate($thumbnail, 100, 100, 100);

  if ($addSquareBorder)
  {
    imagefilledrectangle($thumbnail, 0,0, $thumbnailSize,$thumbnailSize, $white);
    imagerectangle($thumbnail, 0,0, $thumbnailSize-1,$thumbnailSize-1 , $gray);
  }

    // Copy resized image into center of thumbnail
  if ($isTrueColor  &&  function_exists("imagecopyresampled"))
  {
    if (!@imagecopyresampled($thumbnail,$im, $xoffset,$yoffset, 0,0, $thumbnailXSize, $thumbnailYSize, $xs,$ys))
      photoshareImageCopyResampleBicubic($thumbnail, $im, $xoffset,$yoffset, 0,0, $thumbnailXSize, $thumbnailYSize, $xs,$ys);
  }
  else
    imagecopyresized($thumbnail,$im, $xoffset,$yoffset, 0,0, $thumbnailXSize, $thumbnailYSize, $xs,$ys);

    // Save image to file and then copy file into memory buffer (PHP cannot do it directly)

  $tmpdir = pnModGetVar('photoshare', 'tmpdirname');
  $tmpfilename = tempnam($tmpdir, 'IMG');
  if ($tmpfilename === false)
    return photoshareErrorAPI(__FILE__, __LINE__, "Unable to create tmpfile in '$tmpdir' (createThumbnailData)");
  imagepng($thumbnail, $tmpfilename);

  imagedestroy($im);
  imagedestroy($thumbnail);

  if (!($f = fopen($tmpfilename,"rb")))
  {
    @unlink($tmpfilename);
    return photoshareErrorAPI(__FILE__, __LINE__, _PSERRORCREATINGIMAGE . '(2)');
  }

  $thumbnailData = fread($f,filesize($tmpfilename));

  fclose($f);
  @unlink($tmpfilename);

  return $thumbnailData;
}


function photoshareRotateImageFromString($imageString, $direction)
{
    // First create an in-memory rotated image
  $im = imagecreatefromstring($imageString);
  $rotatedIm = photoshareRotateImage($im, $direction);
  imagedestroy($im);

  $tmpdir = pnModGetVar('photoshare', 'tmpdirname');

    // Then have some trouble getting the image back into a string (why ain't there a 'imagepngstring' function?)

    // Copy in-memory to a file
  $tmpfilename = tempnam($tmpdir, 'IMG');
  imagepng($rotatedIm, $tmpfilename);
  imagedestroy($rotatedIm);

  $tmpfilesize = filesize($tmpfilename);
  
    // Load image back into a string
  if (!($f = fopen($tmpfilename, 'rb')))
  {
    unlink($tmpfilename);
    return photoshareErrorAPI(__FILE__, __LINE__, _PSERRORCREATINGIMAGE . "(4,$tmpfilename)");
  }
  $imageData = fread($f, $tmpfilesize);

  fclose($f);
  unlink($tmpfilename);

  return $imageData;
}


function photoshareRotateImage($im, $direction)
{
  $xSize = imagesx($im);
  $ySize = imagesy($im);
  $isTrueColor = false;

  $newIm = photoshareCreateImageDirect($ySize, $xSize, 'PNG', $isTrueColor);

  if ($direction == 'c') // Put test outside loop for efficiency
  {
    for ($x=0; $x<$xSize; ++$x)
    {
      for ($y=0; $y<$ySize; ++$y)
      {
        $newX = $ySize - $y - 1;
        $newY = $x;
  
        imagecopy($newIm,$im, $newX,$newY, $x,$y, 1,1);
      }
    }
  }
  else 
  {
    for ($x=0; $x<$xSize; ++$x)
    {
      for ($y=0; $y<$ySize; ++$y)
      {
        $newX = $y;
        $newY = $xSize - $x - 1;
  
        imagecopy($newIm,$im, $newX,$newY, $x,$y, 1,1);
      }
    }
  }

  return $newIm;
}


// -----------------------------------------------------------------------
// Zip archives
// -----------------------------------------------------------------------
function photoshareAddZipArchive($args)
{
    // Argument check
  if (    !isset($args['folderID'])
      ||  !isset($args['dataFilename'])
      ||  !isset($args['dataFileSize'])
      ||  !isset($args['imageFilename'])
      ||  !isset($args['mimeType']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshareAddZipArchive');

  $folderID      = $args['folderID'];
  $dataFilename  = $args['dataFilename'];
  $dataFileSize  = $args['dataFileSize'];
  $imageFilename = $args['imageFilename'];
  $mimeType      = $args['mimeType'];
  $title         = $args['title'];
  $description   = $args['description'];
  $owner         = pnUserGetVar('uid');

  if (!extension_loaded('zip'))
    return array( array( 'ok' => false,
                         'message' => _PSZIPNOTLOADED ) );


  $tmpfilename = $dataFilename;

  $zip = zip_open($tmpfilename);
  if (!$zip)
  {
    return array( array( 'ok' => false,
                         'message' => _PSZIPOPENERROR . ": $tmpfilename" ) );
  }

  $status = array();

  while ($zipEntry = zip_read($zip))
    $status = array_merge($status, photoshareAddZippedFile($zip, $zipEntry, $folderID));

  zip_close($zip);
  return $status;
}


function photoshareAddZippedFile(&$zip, &$zipEntry, $folderID)
{
    // Read zip info and file data into buffer

  $zipSize = zip_entry_filesize($zipEntry);
  $zipName = zip_entry_name($zipEntry);

  if (!zip_entry_open($zip, $zipEntry, "rb"))
    return array( array( 'ok'      => false,
                         'message' => _PSZIPENTRYOPENERROR . ": $zipName" ) );

  $buffer = zip_entry_read($zipEntry, $zipSize);
  zip_entry_close($zipEntry);

    // Ensure sub-folder exists

    // Split name by slashes into folders and filename and create/verify the folders recursively
  $folders = explode('/', $zipName);
  $subFolderID = photoshareEnsureFolderExists($folderID, $folders, 0);
  if ($subFolderID === false)
    return array( array( 'ok'      => false,
                         'message' => _PSZIPFOLDERCREATEERROR . ": $zipName (" . photoshareErrorAPIGet() . ")" ) );

    // Get actual filename from folderlist (last item in the array)
  $imageName = $folders[sizeof($folders)-1];

    // Create tmp. file and copy image data into it

  $tmpdir = pnModGetVar('photoshare', 'tmpdirname');
  $tmpfilename = tempnam($tmpdir, 'IMG');

  if (!($f=fopen($tmpfilename, "wb")))
  {
    @unlink($tmpfilename);
    return array( array( 'ok'      => false,
                         'message' => _PSZIPTMPOPENERROR . ": $tmpfilename" ) );
  }

  fwrite($f, $buffer);
  fclose($f);

    // Create image (or add recursively zip archive)

  $statusSet = pnModAPIFunc('photoshare',
                            'user',
                            'add_image',
                            array( 'folderID'      => $subFolderID,
                                   'dataFilename'  => $tmpfilename,
                                   'dataFileSize'  => $zipSize,
                                   'imageFilename' => $imageName,
                                   'mimeType'      => 'unknown',
                                   'title'         => '',  // Leave title calculation to API
                                   'description'   => '',
                                   'isUploaded'    => false ));

  @unlink($tmpfilename);

  return $statusSet;
}


function photoshareEnsureFolderExists($parentFolderID, $folders, $folderOffset)
{
    // End of recursion?
  if ($folderOffset == sizeof($folders)-1)
    return $parentFolderID;

  $folderTitle = $folders[$folderOffset];

    // Get ID of existing folder

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

  $sql = "SELECT
            $foldersColumn[id]
          FROM
            $foldersTable
          WHERE $foldersColumn[parentFolder] = '" . pnVarPrepForStore($parentFolderID) . "'
            AND $foldersColumn[title] = '" . pnVarPrepForStore($folderTitle) . "'";

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, "'EnsureFolderExists' failed while executing: $sql");

    // No ID => folder does not exist. Create it.
  if ($result->EOF)
  {
    $folderID = pnModAPIFunc('photoshare',
                             'user',
                             'create_folder',
                             array('parentFolderID' => $parentFolderID,
                                   'title'          => $folderTitle,
                                   'description'    => '',
                                   'topic'          => photoshareGetDefaultTopic(),
                                   'template'       => photoshareGetDefaultTemplate(),
                                   'hidepnframe'    => false,
                                   'blockfromlist'  => false) );

    if (folderID === false)
      return false;
  }
  else
    $folderID = $result->fields[0];

    // Recursive to ensure sub-folders exists
  return photoshareEnsureFolderExists($folderID, $folders, $folderOffset+1);
}

// -----------------------------------------------------------------------
// Import from file
// -----------------------------------------------------------------------

function photoshare_userapi_import_images($args)
{
    // Argument check
  if (   !isset($args['folderID'])
      || !isset($args['directory']))
    return array( array( 'ok'      => false,
                         'message' => _PSMODARGSERRORIN . ' photoshare_userapi_import_images' ) );

  $directory = $args['directory'];
  $folderID  = $args['folderID'];
  $verify    = $args['verify'];

    // Open the supplied directory for reading and check the result
  $dir = @opendir($directory);
  if ($dir === false)
    return array( array( 'ok' => false, 
                         'message' => _PSIMPORTOPENDIRFAILED . " ($directory)") );

    // Store image import result in this array
  $statusSet = array();

    // Store returned result from sub-directories in this
    // (and append it to $statusSet when all local images have been imported)
  $subFolderStatusSet = array();

    // Status for current directory
  $statusSet[] = array( 'ok'      => true,
                        'message' => _PSIMPORTIMPORTING . " $directory" );


    // Read through all the file names
  while (($file=@readdir($dir)) !== false)
  {
    if ($file != '.'  &&  $file != '..'  &&  $file != 'CVS') // Ignore special directories
    {
      $fullFilename = $directory . '/' . $file;

        // Depending on the current file is a file or directory we either import the file
        // or calls ourself recursively with the found sub-folder.

      if (@is_dir($fullFilename))
      {
        $folders = array($file, 'unused-filename');

        if (!$verify)
          $subFolderID = photoshareEnsureFolderExists($folderID, $folders, 0);
        else
          $subFolderID = -1;
        
        if ($subFolderID !== false)
        {
          $tmpStatusSet = pnModAPIFunc('photoshare',
                                       'user',
                                       'import_images',
                                       array( 'folderID'  => $subFolderID,
                                              'directory' => $fullFilename,
                                              'verify'    => $verify));
                                       
          $subFolderStatusSet = array_merge($subFolderStatusSet, $tmpStatusSet);
        }
        else
          $statusSet[] = array( 'ok'      => false,
                                'message' => _PSIMPORTFOLDERCREATEERROR . ": $fullFilename (" . photoshareErrorAPIGet() . ")" );
      }
      else
      if (@is_file($fullFilename)  &&  !$verify)
      {
        $tmpStatusSet = pnModAPIFunc('photoshare',
                                     'user',
                                     'add_image',
                                     array( 'folderID'      => $folderID,
                                            'dataFilename'  => $fullFilename,
                                            'dataFileSize'  => filesize($fullFilename),
                                            'imageFilename' => $file,
                                            'mimeType'      => 'unknown',
                                            'title'         => '',  // Leave title calculation to API
                                            'description'   => '',
                                            'isUploaded'    => false ));

        $statusSet = array_merge($statusSet, $tmpStatusSet);
      }
    }
  }

  @closedir($dir);

    // At last append result from import of sub-folders
  $statusSet = array_merge($statusSet, $subFolderStatusSet);

  return $statusSet;
}


// -----------------------------------------------------------------------
// Misc.
// -----------------------------------------------------------------------

function photoshareGetDefaultTopic()
{
  return pnModGetVar('photoshare', 'defaultTopic');
}


function photoshareGetDefaultTemplate()
{
  return pnModGetVar('photoshare', 'defaultTemplate');
}


// -----------------------------------------------------------------------
// Access control
// -----------------------------------------------------------------------

function photoshare_userapi_get_access($args)
{
    // Argument check
  if (!isset($args['folderID']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_access');

  $folderID = $args['folderID'];

  $accessLists = array();

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

  $sql = "SELECT
            $foldersColumn[accessLevel]
          FROM
            $foldersTable
          WHERE $foldersColumn[id] = '" . pnVarPrepForStore($folderID) . "'";
            
  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_access(1)" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $accessLists['accessLevel'] = $result->fields[0];

  $accessTable = $pntable['photoshare_access'];
  $accessColumn = &$pntable['photoshare_access_column'];

  $sql = "SELECT
            $accessColumn[kind],
            $accessColumn[access],
            $accessColumn[id]
          FROM
            $accessTable
          WHERE
            $accessColumn[folder] = '" . pnVarPrepForStore($folderID) . "'";

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_access(2)" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $groupRead  = array();
  $groupWrite = array();
  $userRead   = array();
  $userWrite  = array();

  for (; !$result->EOF; $result->MoveNext())
  {
    $kind   = $result->fields[0];
    $access = $result->fields[1];
    $id     = $result->fields[2];

    if ($kind == 0  &&  $access == 0)
      $groupRead[$id] = true;
    else if ($kind == 0  &&  $access == 1)
      $groupWrite[$id] = true;
    else if ($kind == 1  &&  $access == 0)
      $userRead[$id] = true;
    else if ($kind == 1  &&  $access == 1)
      $userWrite[$id] = true;
  }

  $accessLists['groupRead']  = $groupRead;
  $accessLists['groupWrite'] = $groupWrite;
  $accessLists['userRead']   = $userRead;
  $accessLists['userWrite']  = $userWrite;

  return $accessLists;
}


function photoshare_userapi_update_access($args)
{
    // Argument check
  if (    !isset($args['folderID'])
      ||  !isset($args['accessLevel'])
      ||  !isset($args['groupReadAccess'])
      ||  !isset($args['groupWriteAccess'])
      ||  !isset($args['userReadAccess'])
      ||  !isset($args['userWriteAccess']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_update_access');

  $folderID         = $args['folderID'];
  $storeFolderID    = pnVarPrepForStore($folderID);
  $accessLevel      = $args['accessLevel'];
  $groupReadAccess  = $args['groupReadAccess'];
  $groupWriteAccess = $args['groupWriteAccess'];
  $userReadAccess   = $args['userReadAccess'];
  $userWriteAccess  = $args['userWriteAccess'];
  $owner            = pnUserGetVar('uid');

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

    // First we remove all existing access records for this folder
  
  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $accessTable = $pntable['photoshare_access'];
  $accessColumn = &$pntable['photoshare_access_column'];
  $foldersTable = $pntable['photoshare_folders'];
  $foldersColumn = &$pntable['photoshare_folders_column'];

    // Delete access control records
  if (!photoshareClearAccessList($storeFolderID))
    return false;

    // Then we set the general access control

  $sql = "UPDATE $foldersTable SET
            $foldersColumn[accessLevel] = " . pnVarPrepForStore($accessLevel) . "
          WHERE $foldersColumn[id] = '" . pnVarPrepForStore($folderID) . "'";

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Update general read/write access" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

    // Now update access control records
  $records = "";
  $first = true;

  photoshareMakeAccessRecord($storeFolderID, $groupReadAccess,  photoshareGroupRead,  $first, $records);
  photoshareMakeAccessRecord($storeFolderID, $groupWriteAccess, photoshareGroupWrite, $first, $records);
  
  if (pnModGetVar('photoshare', 'allowaccessperuser'))
  {
    photoshareMakeAccessRecord($storeFolderID, $userReadAccess,   photoshareUserRead,   $first, $records);
    photoshareMakeAccessRecord($storeFolderID, $userWriteAccess,  photoshareUserWrite,  $first, $records);
  }

  if (!empty($records))
  {
    $sql = "INSERT INTO $accessTable (
              $accessColumn[folder],
              $accessColumn[kind],
              $accessColumn[access],
              $accessColumn[id])
            VALUES $records";

    $dbconn->execute($sql);

    if ($dbconn->errorNo() != 0)
      return photoshareErrorAPI(__FILE__, __LINE__, '"Create access control list" failed: ' . $dbconn->errorMsg() . " while executing: $sql");
  }

  return true;
}


function photoshareMakeAccessRecord($storeFolderID, $list, $kind, &$first, &$records)
{
  foreach ($list as $a)
  {
    $a = trim($a);

    if ($a != '')
    {
      if (!$first)
        $records .= ", ";

      if ($kind == photoshareGroupRead)
        $records .= "('" . $storeFolderID . "', 0,0, '" . pnVarPrepForStore($a) . "')";
      else if ($kind == photoshareGroupWrite)
        $records .= "('" . $storeFolderID . "', 0,1, '" . pnVarPrepForStore($a) . "')";
      else if ($kind == photoshareUserRead)
        $records .= "('" . $storeFolderID . "', 1,0, '" . pnVarPrepForStore($a) . "')";
      else if ($kind == photoshareUserWrite)
        $records .= "('" . $storeFolderID . "', 1,1, '" . pnVarPrepForStore($a) . "')";
      
      $first = false;
    }
  }
}


function photoshareClearAccessList($folderID)
{
  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $accessTable = $pntable['photoshare_access'];
  $accessColumn = &$pntable['photoshare_access_column'];

  $sql = "DELETE FROM 
            $accessTable
          WHERE $accessColumn[folder] = '" . pnVarPrepForStore($folderID) . "'";

  $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"Delete access" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  return true;
}


function photoshare_userapi_get_accessible_folders($args)
{
  if (!isset($args['getForList']))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSMODARGSERRORIN . ' photoshare_userapi_get_accessible_folders');

  $user             = pnVarPrepForStore(isset($args['user']) ? $args['user'] : -1);
  $accessReq        = (isset($args['accessLevel']) ? $args['accessLevel'] : 0);
  $order            = $args['order'];
  $viewKey          = $args['viewKey'];
  $parentFolderID   = $args['parentFolderID'];

    // Check access
  if (!pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ))
    return photoshareErrorAPI(__FILE__, __LINE__, _PSNOAUTH);

  $isAdmin = pnSecAuthAction(0, 'photoshare::', '::', ACCESS_ADMIN);

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = $pntable['photoshare_folders_column'];
  $imagesTable   = $pntable['photoshare_images'];
  $imagesColumn  = $pntable['photoshare_images_column'];
  $accessTable   = $pntable['photoshare_access'];
  $accessColumn  = $pntable['photoshare_access_column'];
  $groupmTable   = $pntable['group_membership'];
  $groupmColumn  = $pntable['group_membership_column'];
  $usersTable    = $pntable['users'];
  $usersColumn   = $pntable['users_column'];
  $topicsTable   = $pntable['topics'];
  $topicsColumn  = $pntable['topics_column'];

  $blockfromlist = ($args['getForList'] ? "AND NOT $foldersColumn[blockfromlist] " : "");
  $topicRestriction = ($args['topic'] ? "$foldersColumn[topic] = " . pnVarPrepForStore($args['topic']) : '');

  $topicsJoin = "";

    // Get order by clause
  $orderBySQL = "";
  if ($order == 'date')
    $orderBySQL = " ORDER BY $foldersColumn[createdDate] DESC";
  else if ($order == 'title')
    $orderBySQL = " ORDER BY $foldersColumn[title]";
  else if ($order == 'owner')
    $orderBySQL = " ORDER BY $usersColumn[uname]";
  else if ($order == 'topic')
  {
    $orderBySQL = " ORDER BY $topicsColumn[topictext]";
    $topicsJoin = "LEFT JOIN $topicsTable ON $topicsColumn[tid] = $foldersColumn[topic]";
  }

    // Open up for use of view key
  $viewKeySQL = (isset($viewKey) 
                  ? "OR $foldersColumn[viewKey] = '" . pnVarPrepForStore($viewKey) ."'"
                  : "");

  if ($isAdmin)
  {
    $whereSQL = '';
    if ($topicRestriction)
      $whereSQL .= "WHERE $topicRestriction";
  }
  else
  {
    if ($user >= 0)
      $whereSQL = "WHERE   ($foldersColumn[owner] = $user\n                OR ";
    else
      $whereSQL = "WHERE (";

    if ($accessReq == photoshareAccessRequirementView)
    {
      $whereSQL .= "$foldersColumn[accessLevel] = " . photoshareFolderAccessViewByAll . "
                OR $foldersColumn[accessLevel] = " . photoshareFolderAccessEditByAll;

      if ($user >= 0)
        $whereSQL .= "
                OR $accessColumn[access] >= 0
                   AND $accessColumn[kind] = 1 
                   AND $accessColumn[id] = $user
                OR $accessColumn[access] >= 0
                   AND $accessColumn[kind] = 0
                   AND $groupmColumn[uid] = $user
                   AND $groupmColumn[gid] = $accessColumn[id]
                $viewKeySQL";
      else
         $whereSQL .= "
                $viewKeySQL";
    }
    else
    {
      $whereSQL .= "$foldersColumn[accessLevel] = " . photoshareFolderAccessEditByAll . " OR ";

      if ($user >= 0)
        $whereSQL .= "
                   $accessColumn[access] >= 1
                   AND $accessColumn[kind] = 1
                   AND $accessColumn[id] = $user
                OR $accessColumn[access] >= 1
                   AND $accessColumn[kind] = 0
                   AND $groupmColumn[uid] = $user
                   AND $groupmColumn[gid] = $accessColumn[id]";
    }
    
    $whereSQL .= ") $blockfromlist";
    
    if ($topicRestriction)
      $whereSQL .= " AND $topicRestriction";
  }

  if (isset($parentFolderID))
    if ($whereSQL == '')
      $whereSQL .= "WHERE $foldersColumn[parentFolder] = " . pnVarPrepForStore($parentFolderID);
    else
      $whereSQL .= " AND $foldersColumn[parentFolder] = " . pnVarPrepForStore($parentFolderID);

  $sql = "SELECT
            $foldersColumn[id],
            $foldersColumn[title]
          FROM
            $foldersTable

               LEFT JOIN $accessTable
                 ON $foldersColumn[id] = $accessColumn[folder]

               LEFT JOIN $groupmTable
                 ON $accessColumn[kind] = 0 AND $groupmColumn[uid] = $user

               LEFT JOIN $usersTable
                 ON $foldersColumn[owner] = $usersColumn[uid]

               $topicsJoin

          $whereSQL

          GROUP BY
            $foldersColumn[id]
            
          $orderBySQL";

  //echo "<pre>$sql</pre>";exit(0);

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_accessible_folders" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  $folders = array();

  for (; !$result->EOF; $result->MoveNext())
  {
    $folders[] = array('id'    => $result->fields[0],
                       'title' => $result->fields[1]);
  }

  //var_dump($folders); exit(0);

  $result->Close();

  return $folders;
}


// -----------------------------------------------------------------------
// User/group information (is this available in the core PN?)
// -----------------------------------------------------------------------

function photoshare_userapi_getallgroups($args)
{
  $getStorageLimit = $args['getStorageLimit'];

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $groupsTable  = $pntable['groups'];
  $groupsColumn = $pntable['groups_column'];
  $setupTable   = $pntable['photoshare_setup'];
  $setupColumn  = $pntable['photoshare_setup_column'];

  if ($getStorageLimit)
  {
    $sql = "SELECT   $groupsColumn[name],
                     $groupsColumn[gid],
                     $setupColumn[storage]
            FROM     $groupsTable
                     LEFT JOIN $setupTable ON $groupsColumn[gid] = $setupColumn[id] 
                                              AND $setupColumn[kind] = " . photoshareGroupSetupKind . "
            ORDER BY $groupsColumn[name]";
  }
  else
  {
    $sql = "SELECT   $groupsColumn[name],
                     $groupsColumn[gid]
            FROM     $groupsTable
            ORDER BY $groupsColumn[name]";
  }

  $result = $dbconn->execute($sql);

  if($dbconn->errorNo() != 0)
    return false;

  if ($result->EOF)
      return false;

  $groups = array();
  while(!$result->EOF)
  {
    if (!empty($result->fields[0]))
      $groups[] = array('name'    => $result->fields[0],
                        'id'      => $result->fields[1],
                        'storage' => ($getStorageLimit ? $result->fields[2] : ''));
    $result->MoveNext();
  }
  $result->Close();

  return $groups;
}


function photoshare_userapi_getallusers($args)
{
  $getStorageLimit = $args['getStorageLimit'];

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $usersTable   = $pntable['users'];
  $usersColumn  = $pntable['users_column'];
  $setupTable   = $pntable['photoshare_setup'];
  $setupColumn  = $pntable['photoshare_setup_column'];

    // This SQL statement concatenates the real name and the user name for sorting, since this allows
    // us to sort users without a real name in the same order as those with a real name.
  if ($getStorageLimit)
  {
    $sql = "SELECT   $usersColumn[name],
                     $usersColumn[uname],
                     $usersColumn[uid],
                     $setupColumn[storage]
            FROM     $usersTable
                     LEFT JOIN $setupTable ON $usersColumn[uid] = $setupColumn[id] 
                                              AND $setupColumn[kind] = " . photoshareUserSetupKind . "
            WHERE    $usersColumn[uname] != 'Anonymous'
            ORDER BY CONCAT($usersColumn[name], $usersColumn[uname])";
  }
  else
  {
    $sql = "SELECT   $usersColumn[name],
                     $usersColumn[uname],
                     $usersColumn[uid]
            FROM     $usersTable
            WHERE    $usersColumn[uname] != 'Anonymous'
            ORDER BY CONCAT($usersColumn[name], $usersColumn[uname])";
  }

  $result = $dbconn->execute($sql);

  if($dbconn->errorNo() != 0)
    return false;

  $users = array();
  while(!$result->EOF)
  {
    $name = (empty($result->fields[0])
             ? $result->fields[1]
             : $result->fields[0] . " (" . $result->fields[1] . ")");

    $users[] = array('name'    => $name,
                     'id'      => $result->fields[2],
                     'storage' => ($getStorageLimit ? $result->fields[3] : ''));
    $result->MoveNext();
  }
  $result->Close();

  return $users;
}


// -----------------------------------------------------------------------
// Templates
// -----------------------------------------------------------------------

function photoshare_userapi_get_templates($args)
{
  $currentTemplate   = $args['currentTemplate'];
  $templates         = array();
  $templateDirectory = "modules/" . photoshareModGetName() . "/templates";

  $dh = opendir($templateDirectory);
  while ($dir = readdir($dh))
  {
    $templateName = $dir;
    $templateDir  = $templateDirectory . '/' . $dir;

    if (is_dir($templateDir) &&
        $dir != '.' &&
        $dir != '..' &&
        $dir != 'CVS')
    {
      $template = photoshare_userapi_load_template( array('templateName' => $templateName) );

      if (!is_bool($template))
      {
        $templateInfo = array( 'id'       => $templateName,
                               'name'     => $template->getTitle(),
                               'selected' => ($templateName == $currentTemplate ));

        $templates[] = $templateInfo;
      }
    }
  }
  closedir($dh);

  return $templates;
}


function photoshare_userapi_load_template($args)
{
  $templateName = $args['templateName'];
  $templateFilename = "modules/" . photoshareModGetName() . "/templates/" . pnVarPrepForOS($templateName) . "/template.php";

  if (file_exists($templateFilename))
  {
    $languageFilename = 'modules/' . photoshareModGetName() . '/templates/' 
                        . pnVarPrepForOS($templateName) . '/lang/' . pnVarPrepForOS(pnUserGetLang()) . '/global.php';

    // Do not complain when unable to load language file
    @include($languageFilename);
    include($templateFilename);

    $templateObjectCreatorName = 'photoshare_template_' . $templateName;
    
    if (function_exists($templateObjectCreatorName))
    {
      $template = $templateObjectCreatorName();
      if ($template->getVersion() > 1)
        return photoshareErrorAPI(__FILE__, __LINE__, "Unsupported template version (" . $template->getVersion() . ") for template '$templateName'");
    }
    return $template;
  }

  return false;
}



// -----------------------------------------------------------------------
// Lists
// -----------------------------------------------------------------------

function photoshare_userapi_get_lists($args)
{
  $currentList   = $args['currentList'];
  $lists         = array();
  $listDirectory = "modules/" . photoshareModGetName() . "/lists";

  $dh = opendir($listDirectory);
  while ($dir = readdir($dh))
  {
    $listName = $dir;
    $listDir  = $listDirectory . '/' . $dir;

    if (is_dir($listDir) &&
        $dir != '.' &&
        $dir != '..' &&
        $dir != 'CVS')
    {
      $list = photoshare_userapi_load_list( array('listName' => $listName) );

      if (!is_bool($list))
      {
        $listInfo = array( 'id'       => $listName,
                           'name'     => $list->getTitle(),
                           'selected' => ($listName == $currentList ));

        $lists[] = $listInfo;
      }
    }
  }
  closedir($dh);

  return $lists;
}


function photoshare_userapi_load_list($args)
{
  if (isset($args['listName']))
    $listName = $args['listName'];
  else
    $listName = pnModGetVar('photoshare', 'mainlist');
  $listFilename = "modules/" . photoshareModGetName() . "/lists/" . pnVarPrepForOS($listName) . "/list.php";

  if (file_exists($listFilename))
  {
    $languageFilename = 'modules/' . photoshareModGetName() . '/lists/' 
                        . pnVarPrepForOS($listName) . '/lang/' . pnVarPrepForOS(pnUserGetLang()) . '/global.php';

    // Do not complain when unable to load language file
    @include($languageFilename);
    include($listFilename);

    $listObjectCreatorName = 'photoshare_list_' . $listName;

    if (function_exists($listObjectCreatorName))
    {
      $list = $listObjectCreatorName();
      if ($list->getVersion() > 1)
        return photoshareErrorAPI(__FILE__, __LINE__, "Unsupported list version (" . $list->getVersion() . ") for list '$listName'");
    }
    else
      $list = false;

    return $list;
  }

  return false;
}


// -----------------------------------------------------------------------
// Access control
// -----------------------------------------------------------------------

function photoshare_userapi_get_image_access($args)
{
  $user      = pnUserGetVar('uid');
  $user      = ($user ? $user : -1);
  $accessReq = (isset($args['accessLevel']) ? $args['accessLevel'] : 0);
  $imageID   = $args['imageID'];
  $viewKey   = $args['viewKey'];

    // admin has access to all.
  if (pnSecAuthAction(0, 'photoshare::', '::', ACCESS_ADMIN))
    return true;

    // Deny access for users that do not even have access to this module
  if (   $accessLevel == photoshareAccessRequirementView  &&  !pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ)
      || $accessLevel == photoshareAccessRequirementEdit  &&  !pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return false;

  list($dbconn) = pnDBGetConn();
  $pntable = pnDBGetTables();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = $pntable['photoshare_folders_column'];
  $imagesTable   = $pntable['photoshare_images'];
  $imagesColumn  = $pntable['photoshare_images_column'];
  $accessTable   = $pntable['photoshare_access'];
  $accessColumn  = $pntable['photoshare_access_column'];
  $groupmTable   = $pntable['group_membership'];
  $groupmColumn  = $pntable['group_membership_column'];
  $usersTable    = $pntable['users'];
  $usersColumn   = $pntable['users_column'];

    // Open up for use of view key
  $viewKeySQL = (isset($viewKey) 
                  ? "OR $foldersColumn[viewKey] = '" . pnVarPrepForStore($viewKey) ."'"
                  : "");

  $whereSQL = "WHERE     $imagesColumn[id] = '" . pnVarPrepForStore($imageID) . "'\n                AND (";

  if ($user >= 0)
    $whereSQL .= "   $foldersColumn[owner] = $user\n                     OR ";

  if ($accessReq == photoshareAccessRequirementView)
  {
    $whereSQL .= "$foldersColumn[accessLevel] = " . photoshareFolderAccessViewByAll . "
                     OR $foldersColumn[accessLevel] = " . photoshareFolderAccessEditByAll;

    if ($user >= 0)
      $whereSQL .= "
                     OR $accessColumn[access] >= 0
                        AND $accessColumn[kind] = 1 
                        AND $accessColumn[id] = $user
                     OR $accessColumn[access] >= 0
                        AND $accessColumn[kind] = 0
                        AND $groupmColumn[uid] = $user
                     $viewKeySQL";
    else
      $whereSQL .= "
                    $viewKeySQL";
  }
  else
  {
    $whereSQL .= "$foldersColumn[accessLevel] = " . photoshareFolderAccessEditByAll . " OR ";

    if ($user >= 0)
      $whereSQL .= "
                 $accessColumn[access] >= 1
                 AND $accessColumn[kind] = 1
                 AND $accessColumn[id] = $user
              OR $accessColumn[access] >= 1
                 AND $accessColumn[kind] = 0
                 AND $groupmColumn[uid] = $user";
  }
  
  $whereSQL .= ")";

  $sql = "SELECT
            COUNT(*)
          FROM
            $imagesTable

               LEFT JOIN $foldersTable
                 ON $foldersColumn[id] = $imagesColumn[parentFolder]

               LEFT JOIN $accessTable
                 ON $foldersColumn[id] = $accessColumn[folder]

               LEFT JOIN $groupmTable
                 ON $groupmColumn[uid] = $user AND $groupmColumn[gid] = $accessColumn[id]

               LEFT JOIN $usersTable
                 ON $foldersColumn[owner] = $usersColumn[uid]

          $whereSQL";

  //echo "<pre>$sql</pre>";exit(0);

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorAPI(__FILE__, __LINE__, '"get_accessible_folders" failed: ' . $dbconn->errorMsg() . " while executing: $sql");

  if ($result->EOF)
    return false;

  $accessOk = $result->fields[0] > 0;
  $result->Close();

  return $accessOk;
}


function photoshareAPIAccessImage($imageID, $accessLevel, $viewKey)
{
  return photoshare_userapi_get_image_access(array('imageID'     => $imageID,
                                                   'accessLevel' => $accessLevel,
                                                   'viewKey'     => $viewKey));
}


function photoshare_userapi_get_folder_access($args)
{
  $folderID     = $args['folderID'];
  $accessLevel  = $args['accessLevel'];
  $viewKey      = $args['viewKey'];

  return photoshareAPIAccessFolder($folderID, $accessLevel, $viewKey);
}


function photoshareAPIAccessFolder($folderID, $accessLevel, $viewKey)
{
    // If user has Admin access then do not look at folder ID - admin has access to all.
  if (pnSecAuthAction(0, 'photoshare::', '::', ACCESS_ADMIN))
    return true;

    // Deny access for users that do not even have access to this module
    // (the check for 'photoshareAccessRequirementAdmin' is relative to the specific folder, so no general check for ADMIN access)
  if (   $accessLevel == photoshareAccessRequirementView  &&  !pnSecAuthAction(0, 'photoshare::', '::', ACCESS_READ)
      || $accessLevel == photoshareAccessRequirementEdit  &&  !pnSecAuthAction(0, 'photoshare::', '::', ACCESS_EDIT))
    return false;

    // No folder specified - access granted
  if ($folderID < 0)
    return true;

    // Now remains the check for access to the actual folder

  if (!pnModAPILoad('photoshare', 'user'))
    return false;

    // First get list of all accessible folders (could be cached)
  static $folderListAvailable = array();
  static $folders = array();

  if (!$folderListAvailable[$accessLevel])
  {
    $f = photoshareAPIAccessGetFolders($accessLevel, $viewKey);
    if ($f == false)
      return false;

    $folders[$accessLevel] = $f;
    $folderListAvailable[$accessLevel] = true;
  }

  return $folders[$accessLevel][$folderID];
}


function photoshareAPIAccessGetFolders($accessLevel, $viewKey)
{
  $user    = pnUserGetVar('uid');
  $folders =  pnModAPIFunc('photoshare',
                           'user',
                           'get_accessible_folders',
                           array('user'        => $user,
                                 'getForList'  => false,
                                 'accessLevel' => $accessLevel,
                                 'viewKey'     => $viewKey) );

  if ($folders == false)
    return false;

    // Convert list to array, mapping folder-IDs to true (making it a set of folderIDs)
  $folderSet = array();
  foreach ($folders as $folder)
    $folderSet[$folder['id']] = true;

  return $folderSet;
}


// -----------------------------------------------------------------------
// Searching
// -----------------------------------------------------------------------

function photoshare_userapi_search(&$args)
{
  if (!isset($args['query']))
    return photoshareErrorApi(__FILE__, __LINE__, "Missing argument 'query' in 'photoshare_userapi_search'");
  if (!isset($args['callback']))
    return photoshareErrorApi(__FILE__, __LINE__, "Missing argument 'callback' in 'photoshare_userapi_search'");

  $query    = $args['query'];
  $match    = $args['match'];
  $callback = &$args['callback'];

  list($dbconn) = pnDBGetConn();
  $pntable      = pnDBGetTables();

    // Split query by whitespace
  $words = preg_split("/\s+/", $query);

  $callback->start();

  $foldersTable  = $pntable['photoshare_folders'];
  $foldersColumn = $pntable['photoshare_folders_column'];
  $imagesTable   = $pntable['photoshare_images'];
  $imagesColumn  = $pntable['photoshare_images_column'];

    // Combine keywords to SQL restriction
  $restriction = '';
  foreach ($words as $word)
  {
    if ($restriction != '')
      $restriction .= ' ' . ($match == 'AND' ? ' AND ' : ' OR ') . ' ';
    $restriction .=   "($imagesColumn[title] LIKE '%" . pnVarPrepForStore($word) . "%' OR "
                    . "$imagesColumn[description] LIKE '%" . pnVarPrepForStore($word) . "%')";
  }
 
  $sql = "SELECT   $foldersColumn[id],
                   $foldersColumn[title],
                   $imagesColumn[id],
                   $imagesColumn[title]
          FROM     $foldersTable
                   LEFT JOIN $imagesTable ON $imagesColumn[parentFolder] = $foldersColumn[id]
          WHERE    $restriction
          ORDER BY $foldersColumn[title]";

  //echo "<pre>$sql</pre>\n"; exit(0);

  $result = $dbconn->execute($sql);

  if ($dbconn->errorNo() != 0)
    return photoshareErrorApi(__FILE__, __LINE__, '"search" failed: ' 
                                                  . $dbconn->errorMsg() . " while executing: $sql");

  for (; !$result->EOF; $result->MoveNext())
  {
    $folderID = $result->fields[0];

    if (photoshareAPIAccessFolder($folderID, photoshareAccessRequirementView, null))
    {
      $folderTitle = $result->fields[1];
      $imageID     = $result->fields[2];
      $imageTitle  = $result->fields[3];

      $url = pnModUrl('photoshare', 'user', 'showimages', array('fid' => $folderID, 'iid' => $imageID));

      $callback->found($folderID, $imageID, $folderTitle, $imageTitle, $url);
    }
  }

  $result->Close();

  $callback->stop();

  return true;
}

?>