<?php
// ----------------------------------------------------------------------
// Copyright (c) 2002-2003 Shawn McKenzie
// http://spidean.mckenzies.net
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// ----------------------------------------------------------------------

define('STARTPHP', str_replace('&lt;', '<', '&lt;?php'));
define('ENDPHP', str_replace('&gt;', '>', '?&gt;'));

//Define path for theme if in multisite directory
if (file_exists(WHERE_IS_PERSO."themes/".$thename."/theme.cfg")) {
    $multipath = WHERE_IS_PERSO;
}
$themepath = $multipath."themes/".$thename."/";
$imgpath = $themepath."images/";

//Get language used
$language = pnUserGetLang();

//Define username if logged in or anonymous if not
if (!pnUserLoggedIn()) {
    $username = _ANONYMOUS;
} else {
    $username = pnUserGetVar('uname');
}
//Define custom templates, colors, image and block display for modules
$module = pnModGetName();

if($custom_module) {
    foreach($custom_module as $name => $innerarray) {
        if($module == $name) {
            //Override main template file names
            foreach($template as $key => $new) {
                if(isset($innerarray[$key])) {
                    $newval[$key] = $innerarray[$key];
                } else {
                    $newval[$key] = $new;
                }
            }
            if($newval) {
                $template = $newval;
                unset($newval);
            }
            //Override default block display
            foreach($block_display as $key => $new) {
                if(isset($innerarray[$key])) {
                    $newval[$key] = $innerarray[$key];
                } else {
                    $newval[$key] = $new;
                }
            }
            if($newval) {
                $block_display = $newval;
                unset($newval);
            }
            //Override style sheet, logo image and colors
            foreach($miscellaneous as $key => $new) {
                if(isset($innerarray[$key])) {
                    $newval[$key] = $innerarray[$key];
                } else {
                    $newval[$key] = $new;
                }
            }
            if($newval) {
                $miscellaneous = $newval;
                unset($newval);
            }
        }
    }
}
//Assign some settings to vars to be used for commands and by modules
$stylesheet = $miscellaneous['stylesheet'];
$logoimg = $miscellaneous['logo'];
$bgcolor1 = $miscellaneous['bgcolor1'];
$bgcolor2 = $miscellaneous['bgcolor2'];
$bgcolor3 = $miscellaneous['bgcolor3'];
$bgcolor4 = $miscellaneous['bgcolor4'];
$textcolor1 = $miscellaneous['textcolor1'];
$textcolor2 = $miscellaneous['textcolor2'];

//Load theme templates into strings
foreach($template as $name => $file) {
    $HTML[$name] = atTemplateLoad($themepath.$file);
}
//Prepare main page and header and footer
//Strip from begining to head or head close
if($strip_head) {
    $HTML['main'] = strip_to($HTML['main'], "/HEAD", null, 0);
} else {
    $HTML['main'] = strip_to($HTML['main'], "HEAD", null, 1);
}
//Strip from body close to end
$HTML['main'] = strip_from($HTML['main'], "/BODY", null, 1);

//Split into themeheader and themefooter
list ($HTML['header'], $HTML['footer']) = split_it($HTML['main'], null, "[modules]");
unset($template, $HTML['main']);

//Break apart on comments, find commands in comments and remove comments
foreach($HTML as $name => $stuff) {
    $output[$name] = atCommandParse($stuff, $command);
}
unset($HTML);

//Add custom style sheet
if($stylesheet && file_exists($themepath."style/".$stylesheet)) {
    $output['header'] = "<link rel=\"StyleSheet\" href=\"".$themepath."style/".$stylesheet."\" type=\"text/css\" />\n".$output['header'];
}
//Add style for languages that read right-to-left
if($language == "ara" || $language == "heb") {
    $output['header'] = "<style type=\"text/css\">body { direction: rtl; text-align: right }</style>\n".$output['header'];
}
//Finish Up
$output['header'] = "<!---------------- Copyright (c) 2002-2003 Shawn McKenzie ----------------->\n\n".$output['header'];
$output['header'] = "\n\n<!------------------- Start AutoTheme Lite Generation --------------------->\n".$output['header'];
$output['footer'] = $output['footer']."\n<!-------------------- End AutoTheme Lite Generation ---------------------->\n\n";

$all_vars = get_defined_vars();

//Functions
//
//Parse templates for commands
function atCommandParse($tmpcontent, $commands)
{
    $content = $tmpcontent;
    foreach($commands as $cmd => $action) {
        $content = str_replace("<!-- $cmd -->", STARTPHP." $action ".ENDPHP, $content);
    }
    foreach($commands as $cmd => $action) {
        $content = str_replace("$cmd", STARTPHP." $action ".ENDPHP, $content);
    }
    return $content;
}

//Find and return list of start and end positions of a tag
//tag string of form "HTML" or "/HTML"
function find_tag($tmpcontent, $tag)
{
    $temp = stristr($tmpcontent, "<".$tag);
    $pos = strlen($tmpcontent) - strlen($temp);
    if($pos) {
        $start = $pos;
        $end = strpos($tmpcontent, ">", $start);
        return array($start, $end);
    } else {
        return false;
    }
}

//Find and return list of start and end positions of a comment
//comment string of form "THIS IS A COMMENT"
function find_comment($tmpcontent, $comment)
{
    while(strpos($tmpcontent, "<!-", $end)) {
        $start = strpos($tmpcontent, "<!-", $end);
        $end = strpos($tmpcontent, "->", $start) + 1;
        $tmpstr = substr($tmpcontent, $start, $end - $start);
        if(stristr($tmpstr, $comment)) {
            return array($start, $end);
        }
    }
    return false;
}

//Split at tag or comment
//tag string of form "HTML" or "/HTML"
//comment string of form "THIS IS A COMMENT"
function split_it($tmpcontent, $tag, $comment)
{
    $first = strip_from($tmpcontent, $tag, $comment, 0);
    $last = strip_to($tmpcontent, $tag, $comment, 1);
    return array($first, $last);
}

//Truncate from tag or comment to end of string
//tag string of form "HTML" or "/HTML"
//comment string of form "THIS IS A COMMENT"
//inc boolean whether to strip the tag its self strip=1
function strip_from($tmpcontent, $tag, $comment, $inc)
{
    if($tag) { //tag was passed
        list ($start, $end) = find_tag($tmpcontent, $tag);
    }
    elseif($comment) { //comment was passed
        list ($start, $end) = find_comment($tmpcontent, $comment);
    }
    if($inc) {
        $len = $start;
    } else {
        $len = $end + 1;
    }
    $newcontent = substr($tmpcontent, 0, $len);
    return $newcontent;
}

//Truncate from begining of string up to tag or comment
//tag string of form "HTML" or "/HTML"
//comment string of form "THIS IS A COMMENT"
//inc boolean whether to strip the tag its self strip=1
function strip_to($tmpcontent, $tag, $comment, $inc)
{
    if($tag) { //tag was passed
        list ($start, $end) = find_tag($tmpcontent, $tag);
    }
    elseif($comment) { //comment was passed
        list ($start, $end) = find_comment($tmpcontent, $comment);
    }
    if($inc) {
        $len = $end + 1;
    } else {
        $len = $start;
    }
    $newcontent = substr($tmpcontent, $len, strlen($tmpcontent) - $len);
    return $newcontent;
}

//Load file contents into string
function atTemplateLoad($filename)
{
	$fp = @fopen($filename, "r");
	if (!($fp)) {
		return 0;
    }
	while (!feof($fp)) {
		$temp .= fread($fp, 4096);
    }
    return $temp;
}

//Render theme part
function atThemeRender($tmpcontent)
{
    global $all_vars;
    extract($all_vars);

    eval(ENDPHP.$tmpcontent);
}

?>
